const char *shader_galdance = {
    "// Based on \"Galactic Dance\" by Sinuousity\n"
    "// https://www.shadertoy.com/view/XtlGWs\n"
    "// License: Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License.\n"
    "\n"
    "// The original source has been modified so it runs in 60 fps on the NVidia Jetson TX2.\n"
    "// More information about changes here: https://www.ignorantus.com/xmasdemo2017/\n"
    "\n"
    "#version 310 es\n"
    "precision highp float;\n"
    "layout(location = 0) in vec2 v_texCoord;\n"
    "layout(location = 0) out vec4 outColor;\n"
    "\n"
    "layout(std140,binding = 0) uniform stuff\n"
    "{\n"
    "    float width, height;\n"
    "    float frame;\n"
    "    float colscale;\n"
    "};\n"
    "\n"
    "#define iTime (frame)\n"
    "\n"
    "#define pi 3.14\n"
    "#define tao 6.28\n"
    "\n"
    "#define overbright 2.0\n"
    "\n"
    "#define armCount 3.0\n"
    "#define armRot 1.6\n"
    "\n"
    "#define innerColor vec4(2.0,0.5,0.1,1.0)\n"
    "#define outerColor vec4(0.8,0.6,1.0,1.0)\n"
    "#define white vec4(1.0,1.0,1.0,1.0)\n"
    "\n"
    "\n"
    "float saturate (in float f)\n"
    "{\n"
    "    return clamp(f,0.0,1.0);\n"
    "}\n"
    "\n"
    "vec4 drawonething( bool nozoom, float offset, float sfac, vec2 texcoord )\n"
    "{\n"
    "    vec2 iResolution = vec2( width, height );\n"
    "\n"
    "    float start = 0.5f - 1.0f/sfac/2.0f;\n"
    "    vec2 fragCoord = (start + texcoord/sfac) * iResolution;\n"
    "\n"
    "//    vec2 fragCoord = ((1.0f - v_texCoord) * iResolution)*0.5f + iResolution*0.25f;\n"
    "\n"
    "    float time = iTime + offset;\n"
    "\n"
    "    vec2 uv = fragCoord.xy / iResolution.xy;\n"
    "    if( !nozoom )\n"
    "        uv = 0.5f + (uv - 0.5f) * (((sin( frame ) + 1.0f) / 2.5f) + 0.25f);\n"
    "\n"
    "    //constant slow rotation\n"
    "    float cost = cos(-time*0.2);\n"
    "    float sint = sin(-time*0.2);\n"
    "    mat2 trm = mat2 (cost,sint,-sint,cost);\n"
    "\n"
    "    //scale 0.0-1.0 uv to -1.0-1.0 p\n"
    "    vec2 p = uv*2.0 - 1.0;\n"
    "    //apply slow rotation\n"
    "    p = p * trm;\n"
    "\n"
    "    //calc distance\n"
    "    float d = length(p);\n"
    "\n"
    "    //build arm rotation matrix\n"
    "    float cosr = cos(armRot*sin(armRot*time));\n"
    "    float sinr = sin(armRot*cos(armRot*time));\n"
    "    mat2 rm = mat2 (cosr,sinr,-sinr,cosr);\n"
    "\n"
    "    //calc arm rotation based on distance\n"
    "    p = mix(p,p * rm,d);\n"
    "\n"
    "    //find angle to middle\n"
    "    float angle = (atan(p.y,p.x)/tao) * 0.5 + 0.5;\n"
    "    //add the crinkle\n"
    "    angle += sin(-time*5.0+fract(d*d*d)*10.0)*0.004;\n"
    "    //calc angle in terms of arm number\n"
    "    angle *= 2.0 * armCount;\n"
    "    angle = fract(angle);\n"
    "    //build arms & wrap the angle around 0.0 & 1.0\n"
    "    float arms = abs(angle*2.0-1.0);\n"
    "    //sharpen arms\n"
    "    arms = pow(arms,10.0*d*d + 5.0);\n"
    "    //calc radial falloff\n"
    "    float bulk = 1.0 - saturate(d);\n"
    "    //create glowy center\n"
    "    float core = pow(bulk,9.0);\n"
    "    //calc color\n"
    "    vec4 color = mix(innerColor,outerColor,d*2.0);\n"
    "\n"
    "    color = bulk * arms * color + core + bulk*0.25*mix(color,white,0.5);\n"
    "    color = vec4( (overbright * color).xyz, colscale );\n"
    "    color = clamp( color, 0.0f, 1.0f );\n"
    "    color.a = max( max( color.r, color.g ), color.b ) * colscale;\n"
    "    return color;\n"
    "}\n"
    "\n"
    "void main( void )\n"
    "{\n"
    "//    vec2 fragCoord = (1.0f - v_texCoord) * iResolution;\n"
    "\n"
    "    float sfac = 1.0f;\n"
    "    if( frame <= 1.3f ) {\n"
    "        sfac = 32.0f;\n"
    "    } else if( frame <= 3.3f ) {\n"
    "        sfac = 32.0f/((frame-1.3f)*16.0f);\n"
    "    } else {\n"
    "        sfac = 1.0f;\n"
    "    }\n"
    "\n"
    "    vec4 pcc = drawonething( false,   0.0f, sfac, v_texCoord                          );\n"
    "\n"
    "\n"
    "    if( frame > 3.3f ) {\n"
    "        float rad = 0.00f + (frame-3.3f)/8.0f;\n"
    "        if( rad > 0.40f ) rad = 0.40f;\n"
    "        float xpos = sin( frame ) * rad;\n"
    "        float ypos = cos( frame ) * rad;\n"
    "\n"
    "        vec4 p00 = drawonething( true,  120.0f, 0.5f, v_texCoord + vec2(  xpos,  ypos ) );\n"
    "        vec4 p11 = drawonething( true,  180.0f, 0.5f, v_texCoord + vec2( -xpos, -ypos ) );\n"
    "        vec4 p01 = drawonething( true,  240.0f, 0.5f, v_texCoord + vec2(  xpos, -ypos ) );\n"
    "        vec4 p10 = drawonething( true,  300.0f, 0.5f, v_texCoord + vec2( -xpos,  ypos ) );\n"
    "        outColor = max( max( max( max( p00, p11 ), p01 ), p10 ), pcc ); // doesn't look as bad as it reads\n"
    "/* sorted\n"
    "        vec4 p0001 = p00.a > p01.a ? p00 : p01;\n"
    "        vec4 p1011 = p10.a > p11.a ? p10 : p11;\n"
    "        vec4 p     = p0001.a > p1011.a ? p0001 : p1011;\n"
    "        outColor = p.a > pcc.a ? p : pcc;\n"
    "*/\n"
    "    } else {\n"
    "        outColor = pcc;\n"
    "    }\n"
    "\n"
    "}\n"
};
const char *shader_glowcity = {
    "// Based on \"Glow City\" by mhnewman\n"
    "// https://www.shadertoy.com/view/XlsyWB\n"
    "// License: Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License.\n"
    "\n"
    "// The original source has been modified so it runs in 60 fps on the NVidia Jetson TX2.\n"
    "// More information about changes here: https://www.ignorantus.com/xmasdemo2017/\n"
    "\n"
    "#version 310 es\n"
    "precision highp float;\n"
    "layout(location = 0) in vec2 v_texCoord;\n"
    "layout(location = 0) out vec4 outColor;\n"
    "\n"
    "layout(std140,binding = 0) uniform stuff\n"
    "{\n"
    "    float width, height;\n"
    "    float frame;\n"
    "    float colscale;\n"
    "};\n"
    "\n"
    "#define iTime (frame)\n"
    "\n"
    "vec2 iResolution = vec2( 1920.0f, 1080.0f );\n"
    "vec3 iMouse = vec3( 0.0f, 0.0f, 0.0f );\n"
    "\n"
    "// Grid marching from iq's Grid of Cylinders\n"
    "// https://www.shadertoy.com/view/4dSGW1\n"
    "\n"
    "const float streetDistance = 0.6;\n"
    "const vec3 streetColor = vec3(4.0, 8.0, 10.0);\n"
    "\n"
    "const float fogDensity = 0.5;\n"
    "const float fogDistance = 4.0;\n"
    "const vec3 fogColor = vec3(0.34, 0.37, 0.4);\n"
    "\n"
    "const float windowSize = 0.075;\n"
    "const float windowSpead = 1.0;\n"
    "\n"
    "const vec3 windowColor = vec3(0.1, 0.2, 1.0);\n"
    "\n"
    "const float beaconProb = 0.0006;\n"
    "const float beaconFreq = 0.6;\n"
    "const vec3 beaconColor = vec3(1.5, 0.2, 0.0);\n"
    "\n"
    "\n"
    "const float tau = 6.283185;\n"
    "\n"
    "float hash1( vec2 p2 )\n"
    "{\n"
    "    p2 = fract(p2 * vec2(5.3983, 5.4427));\n"
    "    p2 += dot(p2.yx, p2.xy + vec2(21.5351, 14.3137));\n"
    "    return fract(p2.x * p2.y * 95.4337);\n"
    "\n"
    "}\n"
    "\n"
    "float hash1( vec2 p2, float p )\n"
    "{\n"
    "    vec3 p3 = fract(vec3(5.3983 * p2.x, 5.4427 * p2.y, 6.9371 * p));\n"
    "    p3 += dot(p3, p3.yzx + 19.19);\n"
    "    return fract((p3.x + p3.y) * p3.z);\n"
    "}\n"
    "\n"
    "vec2 hash2( vec2 p2 )\n"
    "{\n"
    "    return vec2( ( sin( p2.x * p2.y ) + 1.0f ) / 2.0f, cos( p2.x*p2.y ) + 1.0f );\n"
    "}\n"
    "\n"
    "vec2 hash2( vec2 p2, float p )\n"
    "{\n"
    "    return vec2( sin( p2.x ) + 1.0f, cos( p2.y ) * p );\n"
    "}\n"
    "\n"
    "vec3 hash3( vec2 p2 )\n"
    "{\n"
    "    vec3 p3 = fract(vec3(p2.xyx) * vec3(5.3983, 5.4427, 6.9371));\n"
    "    p3 += dot(p3, p3.yxz + 19.19);\n"
    "    return fract((p3.xxy + p3.yzz) * p3.zyx);\n"
    "}\n"
    "\n"
    "float noise1( vec2 p )\n"
    "{\n"
    "    return ( (sin( p.x * 5.78f ) * cos( p.y * 4.23f ) ) + 1.0f ) / 2.0f;\n"
    "}\n"
    "\n"
    "vec4 castRay( vec3 eye, vec3 ray )\n"
    "{\n"
    "    vec2 block = floor(eye.xy);\n"
    "    vec3 ri = 1.0 / ray;\n"
    "    vec3 rs = sign(ray);\n"
    "    vec3 side = 0.5 + 0.5 * rs;\n"
    "    vec2 ris = ri.xy * rs.xy;\n"
    "    vec2 dis = (block - eye.xy + 0.5 + rs.xy * 0.5) * ri.xy;\n"
    "\n"
    "    float beacon = 0.0;\n"
    "\n"
    "    for (int i = 0; i < 100; ++i) {\n"
    "\n"
    "        float height = (0.5 + hash1(block)) * (2.0 + 4.0 * pow(noise1(0.1 * block), 2.5)) * 0.80f + 2.0f;\n"
    "\n"
    "        float dist = 500.0;\n"
    "        float face = 0.0;\n"
    "\n"
    "        vec3 lo = vec3( block + 0.01f + 0.3f * hash2(block, 0.0f), 0.0f );\n"
    "        vec3 hi = vec3( block + 0.69f + 0.3f * hash2(block, 0.5f), height );\n"
    "\n"
    "        vec3 wall = mix(hi, lo, side);\n"
    "        vec3 tw = (wall - eye) * ri;\n"
    "\n"
    "        vec3 dim3 = step(tw.zxy, tw) * step(tw.yzx, tw);\n"
    "\n"
    "        float maxT = dot(dim3, tw);\n"
    "        float maxFace = 1.0 - dim3.z;\n"
    "\n"
    "        vec3 p0 = eye + maxT * ray;\n"
    "        dim3 += step(lo, p0) * step(p0, hi);\n"
    "        if (dim3.x * dim3.y * dim3.z > 0.5 && maxT < dist) {\n"
    "            dist = maxT;\n"
    "            face = maxFace;\n"
    "        }\n"
    "\n"
    "        vec2 h = hash2( block );\n"
    "        if (h.x < beaconProb * (height - 2.0f) * (height - 2.0f) ) {\n"
    "            vec3 center = vec3(block + 0.5, height + 0.2);\n"
    "            float t = dot(center - eye, ray);\n"
    "            if (t < dist) {\n"
    "                vec3 p = eye + t * ray;\n"
    "                float fog = (exp(-p.z / fogDistance) - exp(-eye.z / fogDistance)) / ray.z;\n"
    "                fog = exp(fogDensity * fog);\n"
    "\n"
    "                t = distance(center, p);\n"
    "                fog *= smoothstep(1.0, 0.5, cos(tau * (beaconFreq * iTime + h.y)));\n"
    "                beacon += fog * pow(clamp(1.0 - 2.0 * t, 0.0, 1.0), 2.0);\n"
    "            }\n"
    "        }\n"
    "\n"
    "        if (dist < 400.0) { // 400.0f\n"
    "            return vec4(dist, beacon, face, 1.0);\n"
    "        }\n"
    "\n"
    "        float t = eye.z * ri.z;\n"
    "        vec3 p = eye - t * ray;\n"
    "        vec2 g = p.xy - block;\n"
    "        if (g.x > 0.0 && g.x < 1.0 && g.y > 0.0 && g.y < 1.0) {\n"
    "            return vec4(-t, beacon, 0.0, 1.0);\n"
    "        }\n"
    "\n"
    "        vec2 dim = step(dis.xy, dis.yx);\n"
    "        dis += dim * ris;\n"
    "        block += dim * rs.xy;\n"
    "    }\n"
    "\n"
    "    if (ray.z < 0.0) {\n"
    "        return vec4(-eye.z * ri.z, beacon, 0.0, 1.0);\n"
    "    }\n"
    "\n"
    "    return vec4(0.0, beacon, 0.0, 0.0);\n"
    "}\n"
    "\n"
    "void main( void )\n"
    "{\n"
    "    vec2 fragCoord = (1.0f - v_texCoord) * iResolution;\n"
    "\n"
    "    vec2 m = vec2(0.03 * iTime, 0.8);\n"
    "    if (iMouse.z > 0.0)\n"
    "        m = iMouse.xy / iResolution.xy;\n"
    "    m *= tau * vec2(1.0, 0.25);\n"
    "\n"
    "    vec3 center = vec3(6.0 * iTime, 0.5, 3.0);\n"
    "    float dist = 20.0;\n"
    "    vec3 eye = center + vec3(dist * sin(m.x) * sin(m.y), dist * cos(m.x) * sin(m.y), dist * cos(m.y));\n"
    "    float zoom = 3.0;\n"
    "\n"
    "    vec3 forward = normalize(center - eye);\n"
    "    vec3 right = normalize(cross(forward, vec3(0.0, 0.0, 1.0)));\n"
    "    vec3 up = cross(right, forward);\n"
    "    vec2 xy = 2.0 * fragCoord - iResolution.xy;\n"
    "    zoom *= iResolution.y;\n"
    "    vec3 ray = normalize(xy.x * right + xy.y * up + zoom * forward);\n"
    "\n"
    "    vec4 res = castRay(eye, ray);\n"
    "    if (res.w < 0.5) {\n"
    "        outColor = vec4(fogColor + res.y * beaconColor, colscale);\n"
    "        return;\n"
    "    }\n"
    "\n"
    "    vec3 p = eye + res.x * ray;\n"
    "\n"
    "    vec2 block = floor(p.xy);\n"
    "    vec3 window = floor(p / windowSize);\n"
    "    float x = hash1(block, window.x);\n"
    "    float y = hash1(block, window.y);\n"
    "    float z = hash1(block, window.z);\n"
    "    vec3 color = windowColor + windowSpead * (hash3(block) - 0.5);\n"
    "    color *= smoothstep(0.1, 0.9, fract(2.5 * (x * y * z)));\n"
    "    color += streetColor * exp(-p.z / streetDistance);\n"
    "\n"
    "    vec3 level = 0.25 * streetColor * exp(-4.0 * p.z / streetDistance);\n"
    "    color = clamp(mix(level, color, res.z), 0.0, 1.0);\n"
    "\n"
    "    float fog = (exp(-p.z / fogDistance) - exp(-eye.z / fogDistance)) / ray.z;\n"
    "    fog = exp(fogDensity * fog);\n"
    "    color = mix(fogColor, color, fog);\n"
    "\n"
    "    color += res.y * beaconColor;\n"
    "    color += pow(res.y, 2.0);\n"
    "\n"
    "    outColor = vec4(color, colscale);\n"
    "}\n"
};
const char *shader_tracer = {
    "// GPURay 2.0: Raytracer fragment shader.\n"
    "// Written by Nils L. Corneliusen.\n"
    "// License: CC0 1.0 Universal (CC0 1.0) Public Domain Dedication license\n"
    "// Please refer to the article at https://www.ignorantus.com/gpuray/ for more information.\n"
    "// Used in the Xmas demo 2017: https://www.ignorantus.com/xmasdemo2017/\n"
    "\n"
    "#version 310 es\n"
    "precision highp float;\n"
    "layout(location = 0) in vec2 v_texCoord;\n"
    "layout(location = 0) out vec4 outColor;\n"
    "\n"
    "#define OBJNUM (4*32)\n"
    "#define REFLECTIONS 3\n"
    "\n"
    "#define FARDIST 256.0f\n"
    "\n"
    "// must hardcode max, since uniform indeterminate not supported\n"
    "#define MAXW 1920\n"
    "#define MAXH 1080\n"
    "\n"
    "#define BLOCKW 16\n"
    "#define BLOCKH 16\n"
    "\n"
    "#define MAXWR ((MAXW+BLOCKW-1)&~(BLOCKW-1))\n"
    "#define MAXHR ((MAXH+BLOCKH-1)&~(BLOCKH-1))\n"
    "\n"
    "#define BLOCKSX (MAXWR/BLOCKW)\n"
    "#define BLOCKSY (MAXHR/BLOCKH)\n"
    "\n"
    "#define BLOCK_ANY 0\n"
    "#define BLOCK_BG  1\n"
    "#define BLOCK_SPH 2\n"
    "\n"
    "//#pragma optionNV (unroll all)\n"
    "\n"
    "layout(std140,binding = 0) uniform stuff\n"
    "{\n"
    "    float width, height;\n"
    "    float frame;\n"
    "    float colscale;\n"
    "    vec4 obj_pos[OBJNUM];\n"
    "    vec4 obj_col[OBJNUM];\n"
    "    vec4 plane_norm;\n"
    "    vec4 plane_col;\n"
    "    vec4 eye;\n"
    "    vec4 light_pos;\n"
    "    vec4 startpos;\n"
    "    float ax;\n"
    "    float ay;\n"
    "    uvec4 probe[BLOCKSX*BLOCKSY/4/4];\n"
    "};\n"
    "\n"
    "\n"
    "void main()\n"
    "{\n"
    "    vec3 ray_dir = normalize( vec3( startpos.x + ax * v_texCoord.x,\n"
    "                                    startpos.y + ay * v_texCoord.y,\n"
    "                                    startpos.z ) );\n"
    "\n"
    "    vec3 col = vec3( 0.0f );\n"
    "\n"
    "    vec3 ray_pos = eye.xyz;\n"
    "\n"
    "	// Find probe val...\n"
    "    int blx = int(floor(v_texCoord.x*width /float(BLOCKW)));\n"
    "    int bly = int(floor(v_texCoord.y*height/float(BLOCKH)));\n"
    "    int offset = bly*BLOCKSX+blx;\n"
    "    uint val = probe[offset/4/4][(offset/4)&0x03];\n"
    "    int prval = int((val>>((offset&0x03)<<3))&uint(0xff));\n"
    "\n"
    "    if( v_texCoord.x < 1.0f - colscale ) {\n"
    "\n"
    "        vec3 col = vec3( 0.0f, 0.0f, 0.0f );\n"
    "\n"
    "        if( prval == BLOCK_SPH )\n"
    "            col = vec3( 1.0f, 0.0f, 0.0f );\n"
    "        else if( prval == BLOCK_BG )\n"
    "            col = vec3( 0.5f, 0.5f, 0.5f );\n"
    "        else if( prval == BLOCK_ANY )\n"
    "            col = vec3( 0.0f, 0.0f, 1.0f );\n"
    "\n"
    "        outColor = vec4( col,  colscale );\n"
    "        return;\n"
    "\n"
    "    }\n"
    "\n"
    "    if( prval == BLOCK_SPH || prval == BLOCK_ANY ) {\n"
    "\n"
    "        int refl;\n"
    "        for( refl = 0; refl < REFLECTIONS; refl++ ) {\n"
    "\n"
    "            float rt_dist = 0.0f;\n"
    "            int rt_i = -1;\n"
    "            vec3 rt_t0;\n"
    "\n"
    "            vec4 v0, v1, v2, v3;\n"
    "\n"
    "            // preload 0-3\n"
    "            v0 = vec4( ray_pos - obj_pos[0].xyz, obj_pos[0].w );\n"
    "            v1 = vec4( ray_pos - obj_pos[1].xyz, obj_pos[1].w );\n"
    "            v2 = vec4( ray_pos - obj_pos[2].xyz, obj_pos[2].w );\n"
    "            v3 = vec4( ray_pos - obj_pos[3].xyz, obj_pos[3].w );\n"
    "\n"
    "            // trace_ray()\n"
    "            for( int i = 0; i < OBJNUM; i += 32 ) {\n"
    "                vec4 v4, v5, v6, v7;\n"
    "                vec4 b0, d0;\n"
    "\n"
    "                // preload 4-7\n"
    "                v4 = vec4( ray_pos - obj_pos[i+4].xyz, obj_pos[i+4].w );\n"
    "                v5 = vec4( ray_pos - obj_pos[i+5].xyz, obj_pos[i+5].w );\n"
    "                v6 = vec4( ray_pos - obj_pos[i+6].xyz, obj_pos[i+6].w );\n"
    "                v7 = vec4( ray_pos - obj_pos[i+7].xyz, obj_pos[i+7].w );\n"
    "\n"
    "                // calc 0-3\n"
    "                b0 = vec4( dot( v0.xyz, ray_dir ),\n"
    "                           dot( v1.xyz, ray_dir ),\n"
    "                           dot( v2.xyz, ray_dir ),\n"
    "                           dot( v3.xyz, ray_dir ) );\n"
    "\n"
    "                d0 = vec4( b0.x * b0.x - dot( v0.xyz, v0.xyz ) + v0.w,\n"
    "                           b0.y * b0.y - dot( v1.xyz, v1.xyz ) + v1.w,\n"
    "                           b0.z * b0.z - dot( v2.xyz, v2.xyz ) + v2.w,\n"
    "                           b0.w * b0.w - dot( v3.xyz, v3.xyz ) + v3.w );\n"
    "\n"
    "                if( any( greaterThan( d0, vec4( 0.0f ) ) ) ) {\n"
    "\n"
    "                    vec4  t0 = 1.0f / (-b0 - sqrt( d0 ));\n"
    "                    float mv = max( max( t0.x, t0.y ), max( t0.z, t0.w ) );\n"
    "\n"
    "                    if( mv > rt_dist ) {\n"
    "                        rt_dist = mv;\n"
    "                        rt_t0   = t0.yzw;\n"
    "                        rt_i    = i + 0;\n"
    "                    }\n"
    "\n"
    "                }\n"
    "\n"
    "                // preload 8-11\n"
    "                v0 = vec4( ray_pos - obj_pos[i+ 8].xyz, obj_pos[i+ 8].w );\n"
    "                v1 = vec4( ray_pos - obj_pos[i+ 9].xyz, obj_pos[i+ 9].w );\n"
    "                v2 = vec4( ray_pos - obj_pos[i+10].xyz, obj_pos[i+10].w );\n"
    "                v3 = vec4( ray_pos - obj_pos[i+11].xyz, obj_pos[i+11].w );\n"
    "\n"
    "                // calc 4-7\n"
    "                b0 = vec4( dot( v4.xyz, ray_dir ),\n"
    "                           dot( v5.xyz, ray_dir ),\n"
    "                           dot( v6.xyz, ray_dir ),\n"
    "                           dot( v7.xyz, ray_dir ) );\n"
    "\n"
    "                d0 = vec4( b0.x * b0.x - dot( v4.xyz, v4.xyz ) + v4.w,\n"
    "                           b0.y * b0.y - dot( v5.xyz, v5.xyz ) + v5.w,\n"
    "                           b0.z * b0.z - dot( v6.xyz, v6.xyz ) + v6.w,\n"
    "                           b0.w * b0.w - dot( v7.xyz, v7.xyz ) + v7.w );\n"
    "\n"
    "                if( any( greaterThan( d0, vec4( 0.0f ) ) ) ) {\n"
    "\n"
    "                    vec4  t0 = 1.0f / (-b0 - sqrt( d0 ));\n"
    "                    float mv = max( max( t0.x, t0.y ), max( t0.z, t0.w ) );\n"
    "\n"
    "                    if( mv > rt_dist ) {\n"
    "                        rt_dist = mv;\n"
    "                        rt_t0   = t0.yzw;\n"
    "                        rt_i    = i + 4;\n"
    "                    }\n"
    "\n"
    "                }\n"
    "\n"
    "                // preload 12-15\n"
    "                v4 = vec4( ray_pos - obj_pos[i+12].xyz, obj_pos[i+12].w );\n"
    "                v5 = vec4( ray_pos - obj_pos[i+13].xyz, obj_pos[i+13].w );\n"
    "                v6 = vec4( ray_pos - obj_pos[i+14].xyz, obj_pos[i+14].w );\n"
    "                v7 = vec4( ray_pos - obj_pos[i+15].xyz, obj_pos[i+15].w );\n"
    "\n"
    "                // calc 8-11\n"
    "                b0 = vec4( dot( v0.xyz, ray_dir ),\n"
    "                           dot( v1.xyz, ray_dir ),\n"
    "                           dot( v2.xyz, ray_dir ),\n"
    "                           dot( v3.xyz, ray_dir ) );\n"
    "\n"
    "                d0 = vec4( b0.x * b0.x - dot( v0.xyz, v0.xyz ) + v0.w,\n"
    "                           b0.y * b0.y - dot( v1.xyz, v1.xyz ) + v1.w,\n"
    "                           b0.z * b0.z - dot( v2.xyz, v2.xyz ) + v2.w,\n"
    "                           b0.w * b0.w - dot( v3.xyz, v3.xyz ) + v3.w );\n"
    "\n"
    "                if( any( greaterThan( d0, vec4( 0.0f ) ) ) ) {\n"
    "\n"
    "                    vec4  t0 = 1.0f / (-b0 - sqrt( d0 ));\n"
    "                    float mv = max( max( t0.x, t0.y ), max( t0.z, t0.w ) );\n"
    "\n"
    "                    if( mv > rt_dist ) {\n"
    "                        rt_dist = mv;\n"
    "                        rt_t0   = t0.yzw;\n"
    "                        rt_i    = i + 8;\n"
    "                    }\n"
    "\n"
    "                }\n"
    "\n"
    "                // preload 16-19\n"
    "                v0 = vec4( ray_pos - obj_pos[i+16].xyz, obj_pos[i+16].w );\n"
    "                v1 = vec4( ray_pos - obj_pos[i+17].xyz, obj_pos[i+17].w );\n"
    "                v2 = vec4( ray_pos - obj_pos[i+18].xyz, obj_pos[i+18].w );\n"
    "                v3 = vec4( ray_pos - obj_pos[i+19].xyz, obj_pos[i+19].w );\n"
    "\n"
    "                // calc 12-15\n"
    "                b0 = vec4( dot( v4.xyz, ray_dir ),\n"
    "                           dot( v5.xyz, ray_dir ),\n"
    "                           dot( v6.xyz, ray_dir ),\n"
    "                           dot( v7.xyz, ray_dir ) );\n"
    "\n"
    "                d0 = vec4( b0.x * b0.x - dot( v4.xyz, v4.xyz ) + v4.w,\n"
    "                           b0.y * b0.y - dot( v5.xyz, v5.xyz ) + v5.w,\n"
    "                           b0.z * b0.z - dot( v6.xyz, v6.xyz ) + v6.w,\n"
    "                           b0.w * b0.w - dot( v7.xyz, v7.xyz ) + v7.w );\n"
    "\n"
    "                if( any( greaterThan( d0, vec4( 0.0f ) ) ) ) {\n"
    "\n"
    "                    vec4  t0 = 1.0f / (-b0 - sqrt( d0 ));\n"
    "                    float mv = max( max( t0.x, t0.y ), max( t0.z, t0.w ) );\n"
    "\n"
    "                    if( mv > rt_dist ) {\n"
    "                        rt_dist = mv;\n"
    "                        rt_t0   = t0.yzw;\n"
    "                        rt_i    = i + 12;\n"
    "                    }\n"
    "\n"
    "                }\n"
    "\n"
    "                // preload 20-23\n"
    "                v4 = vec4( ray_pos - obj_pos[i+20].xyz, obj_pos[i+20].w );\n"
    "                v5 = vec4( ray_pos - obj_pos[i+21].xyz, obj_pos[i+21].w );\n"
    "                v6 = vec4( ray_pos - obj_pos[i+22].xyz, obj_pos[i+22].w );\n"
    "                v7 = vec4( ray_pos - obj_pos[i+23].xyz, obj_pos[i+23].w );\n"
    "\n"
    "                // calc 16-19\n"
    "                b0 = vec4( dot( v0.xyz, ray_dir ),\n"
    "                           dot( v1.xyz, ray_dir ),\n"
    "                           dot( v2.xyz, ray_dir ),\n"
    "                           dot( v3.xyz, ray_dir ) );\n"
    "\n"
    "                d0 = vec4( b0.x * b0.x - dot( v0.xyz, v0.xyz ) + v0.w,\n"
    "                           b0.y * b0.y - dot( v1.xyz, v1.xyz ) + v1.w,\n"
    "                           b0.z * b0.z - dot( v2.xyz, v2.xyz ) + v2.w,\n"
    "                           b0.w * b0.w - dot( v3.xyz, v3.xyz ) + v3.w );\n"
    "\n"
    "                if( any( greaterThan( d0, vec4( 0.0f ) ) ) ) {\n"
    "\n"
    "                    vec4  t0 = 1.0f / (-b0 - sqrt( d0 ));\n"
    "                    float mv = max( max( t0.x, t0.y ), max( t0.z, t0.w ) );\n"
    "\n"
    "                    if( mv > rt_dist ) {\n"
    "                        rt_dist = mv;\n"
    "                        rt_t0   = t0.yzw;\n"
    "                        rt_i    = i + 16;\n"
    "                    }\n"
    "\n"
    "                }\n"
    "\n"
    "                // preload 24-27\n"
    "                v0 = vec4( ray_pos - obj_pos[i+24].xyz, obj_pos[i+24].w );\n"
    "                v1 = vec4( ray_pos - obj_pos[i+25].xyz, obj_pos[i+25].w );\n"
    "                v2 = vec4( ray_pos - obj_pos[i+26].xyz, obj_pos[i+26].w );\n"
    "                v3 = vec4( ray_pos - obj_pos[i+27].xyz, obj_pos[i+27].w );\n"
    "\n"
    "                // calc 20-23\n"
    "                b0 = vec4( dot( v4.xyz, ray_dir ),\n"
    "                           dot( v5.xyz, ray_dir ),\n"
    "                           dot( v6.xyz, ray_dir ),\n"
    "                           dot( v7.xyz, ray_dir ) );\n"
    "\n"
    "                d0 = vec4( b0.x * b0.x - dot( v4.xyz, v4.xyz ) + v4.w,\n"
    "                           b0.y * b0.y - dot( v5.xyz, v5.xyz ) + v5.w,\n"
    "                           b0.z * b0.z - dot( v6.xyz, v6.xyz ) + v6.w,\n"
    "                           b0.w * b0.w - dot( v7.xyz, v7.xyz ) + v7.w );\n"
    "\n"
    "                if( any( greaterThan( d0, vec4( 0.0f ) ) ) ) {\n"
    "\n"
    "                    vec4  t0 = 1.0f / (-b0 - sqrt( d0 ));\n"
    "                    float mv = max( max( t0.x, t0.y ), max( t0.z, t0.w ) );\n"
    "\n"
    "                    if( mv > rt_dist ) {\n"
    "                        rt_dist = mv;\n"
    "                        rt_t0   = t0.yzw;\n"
    "                        rt_i    = i + 20;\n"
    "                    }\n"
    "\n"
    "                }\n"
    "\n"
    "                // preload 28-31\n"
    "                v4 = vec4( ray_pos - obj_pos[i+28].xyz, obj_pos[i+28].w );\n"
    "                v5 = vec4( ray_pos - obj_pos[i+29].xyz, obj_pos[i+29].w );\n"
    "                v6 = vec4( ray_pos - obj_pos[i+30].xyz, obj_pos[i+30].w );\n"
    "                v7 = vec4( ray_pos - obj_pos[i+31].xyz, obj_pos[i+31].w );\n"
    "\n"
    "                // calc 24-27\n"
    "                b0 = vec4( dot( v0.xyz, ray_dir ),\n"
    "                           dot( v1.xyz, ray_dir ),\n"
    "                           dot( v2.xyz, ray_dir ),\n"
    "                           dot( v3.xyz, ray_dir ) );\n"
    "\n"
    "                d0 = vec4( b0.x * b0.x - dot( v0.xyz, v0.xyz ) + v0.w,\n"
    "                           b0.y * b0.y - dot( v1.xyz, v1.xyz ) + v1.w,\n"
    "                           b0.z * b0.z - dot( v2.xyz, v2.xyz ) + v2.w,\n"
    "                           b0.w * b0.w - dot( v3.xyz, v3.xyz ) + v3.w );\n"
    "\n"
    "                if( any( greaterThan( d0, vec4( 0.0f ) ) ) ) {\n"
    "\n"
    "                    vec4  t0 = 1.0f / (-b0 - sqrt( d0 ));\n"
    "                    float mv = max( max( t0.x, t0.y ), max( t0.z, t0.w ) );\n"
    "\n"
    "                    if( mv > rt_dist ) {\n"
    "                        rt_dist = mv;\n"
    "                        rt_t0   = t0.yzw;\n"
    "                        rt_i    = i + 24;\n"
    "                    }\n"
    "\n"
    "                }\n"
    "\n"
    "                // preload 0-3 next iteration\n"
    "                v0 = vec4( ray_pos - obj_pos[i+32].xyz, obj_pos[i+32].w );\n"
    "                v1 = vec4( ray_pos - obj_pos[i+33].xyz, obj_pos[i+33].w );\n"
    "                v2 = vec4( ray_pos - obj_pos[i+34].xyz, obj_pos[i+34].w );\n"
    "                v3 = vec4( ray_pos - obj_pos[i+35].xyz, obj_pos[i+35].w );\n"
    "\n"
    "                // calc 28-31\n"
    "                b0 = vec4( dot( v4.xyz, ray_dir ),\n"
    "                           dot( v5.xyz, ray_dir ),\n"
    "                           dot( v6.xyz, ray_dir ),\n"
    "                           dot( v7.xyz, ray_dir ) );\n"
    "\n"
    "                d0 = vec4( b0.x * b0.x - dot( v4.xyz, v4.xyz ) + v4.w,\n"
    "                           b0.y * b0.y - dot( v5.xyz, v5.xyz ) + v5.w,\n"
    "                           b0.z * b0.z - dot( v6.xyz, v6.xyz ) + v6.w,\n"
    "                           b0.w * b0.w - dot( v7.xyz, v7.xyz ) + v7.w );\n"
    "\n"
    "                if( any( greaterThan( d0, vec4( 0.0f ) ) ) ) {\n"
    "\n"
    "                    vec4  t0 = 1.0f / (-b0 - sqrt( d0 ));\n"
    "                    float mv = max( max( t0.x, t0.y ), max( t0.z, t0.w ) );\n"
    "\n"
    "                    if( mv > rt_dist ) {\n"
    "                        rt_dist = mv;\n"
    "                        rt_t0   = t0.yzw;\n"
    "                        rt_i    = i + 28;\n"
    "                    }\n"
    "\n"
    "                }\n"
    "\n"
    "                if( rt_dist > 0.33f ) break;\n"
    "            }\n"
    "\n"
    "            if( rt_i == -1 ) break;\n"
    "\n"
    "            ivec3 iv = ivec3( equal( rt_t0, vec3( rt_dist ) ) );\n"
    "            rt_i += iv.x + iv.y + iv.y + iv.z + iv.z + iv.z;\n"
    "\n"
    "            // sphere hit + reflection\n"
    "            ray_pos += ray_dir * (1.0f / rt_dist);\n"
    "            vec3 n   = normalize( ray_pos - obj_pos[rt_i].xyz );\n"
    "            vec3 l   = normalize( light_pos.xyz - ray_pos );\n"
    "\n"
    "            float diffuse  = max( dot( n, l ), 0.0f );\n"
    "            float specular = pow( max( dot( ray_dir, l - n * 2.0f * diffuse ), 0.0f ), 8.0f );\n"
    "\n"
    "            col += vec3( specular ) + obj_col[rt_i].xyz * diffuse;\n"
    "            ray_dir = reflect( ray_dir, n );\n"
    "\n"
    "        }\n"
    "\n"
    "        if( refl != 0 ) {\n"
    "            outColor = vec4( col, colscale );\n"
    "            return;\n"
    "        }\n"
    "\n"
    "    }\n"
    "\n"
    "    // plane/sky\n"
    "    float rt_dist = -(dot( ray_pos, plane_norm.xyz ) + plane_norm.w) / dot( ray_dir, plane_norm.xyz );\n"
    "\n"
    "    if( rt_dist <= 0.0f ) {\n"
    "        // plane miss, sky\n"
    "        vec2 dxy = v_texCoord - 0.5f;\n"
    "        float dv = 1.0f - sqrt( dot( dxy, dxy ) + (sin(fract(frame/2.0f)*2.0f*3.14f)+1.0f)*0.5f );\n"
    "        outColor = vec4( plane_col.yzx * dv, colscale );\n"
    "        return;\n"
    "    }\n"
    "\n"
    "    //  plane hit\n"
    "    ray_pos += ray_dir * rt_dist;\n"
    "    ray_dir  = normalize( light_pos.xyz - ray_pos );\n"
    "\n"
    "    // check shadow\n"
    "    vec4 v0, v1, v2, v3;\n"
    "\n"
    "    // preload 0-3\n"
    "    v0 = vec4( ray_pos - obj_pos[0].xyz, obj_pos[0].w );\n"
    "    v1 = vec4( ray_pos - obj_pos[1].xyz, obj_pos[1].w );\n"
    "    v2 = vec4( ray_pos - obj_pos[2].xyz, obj_pos[2].w );\n"
    "    v3 = vec4( ray_pos - obj_pos[3].xyz, obj_pos[3].w );\n"
    "\n"
    "    int i;\n"
    "    for( i = 0; i < OBJNUM; i += 16 ) {\n"
    "        vec4 v4, v5, v6, v7;\n"
    "        vec4 d0, d1, d2, d3;\n"
    "        vec4 b0;\n"
    "\n"
    "        // preload 4-7\n"
    "        v4 = vec4( ray_pos - obj_pos[i+4].xyz, obj_pos[i+4].w );\n"
    "        v5 = vec4( ray_pos - obj_pos[i+5].xyz, obj_pos[i+5].w );\n"
    "        v6 = vec4( ray_pos - obj_pos[i+6].xyz, obj_pos[i+6].w );\n"
    "        v7 = vec4( ray_pos - obj_pos[i+7].xyz, obj_pos[i+7].w );\n"
    "\n"
    "        // calc 0-3\n"
    "        b0 = vec4( dot( v0.xyz, ray_dir ),\n"
    "                   dot( v1.xyz, ray_dir ),\n"
    "                   dot( v2.xyz, ray_dir ),\n"
    "                   dot( v3.xyz, ray_dir ) );\n"
    "\n"
    "        d0 = vec4( b0.x * b0.x - dot( v0.xyz, v0.xyz ) + v0.w,\n"
    "                   b0.y * b0.y - dot( v1.xyz, v1.xyz ) + v1.w,\n"
    "                   b0.z * b0.z - dot( v2.xyz, v2.xyz ) + v2.w,\n"
    "                   b0.w * b0.w - dot( v3.xyz, v3.xyz ) + v3.w );\n"
    "\n"
    "        // preload 8-11\n"
    "        v0 = vec4( ray_pos - obj_pos[i+ 8].xyz, obj_pos[i+ 8].w );\n"
    "        v1 = vec4( ray_pos - obj_pos[i+ 9].xyz, obj_pos[i+ 9].w );\n"
    "        v2 = vec4( ray_pos - obj_pos[i+10].xyz, obj_pos[i+10].w );\n"
    "        v3 = vec4( ray_pos - obj_pos[i+11].xyz, obj_pos[i+11].w );\n"
    "\n"
    "        // calc 4-7\n"
    "        b0 = vec4( dot( v4.xyz, ray_dir ),\n"
    "                   dot( v5.xyz, ray_dir ),\n"
    "                   dot( v6.xyz, ray_dir ),\n"
    "                   dot( v7.xyz, ray_dir ) );\n"
    "\n"
    "        d1 = vec4( b0.x * b0.x - dot( v4.xyz, v4.xyz ) + v4.w,\n"
    "                   b0.y * b0.y - dot( v5.xyz, v5.xyz ) + v5.w,\n"
    "                   b0.z * b0.z - dot( v6.xyz, v6.xyz ) + v6.w,\n"
    "                   b0.w * b0.w - dot( v7.xyz, v7.xyz ) + v7.w );\n"
    "\n"
    "        // preload 12-15\n"
    "        v4 = vec4( ray_pos - obj_pos[i+12].xyz, obj_pos[i+12].w );\n"
    "        v5 = vec4( ray_pos - obj_pos[i+13].xyz, obj_pos[i+13].w );\n"
    "        v6 = vec4( ray_pos - obj_pos[i+14].xyz, obj_pos[i+14].w );\n"
    "        v7 = vec4( ray_pos - obj_pos[i+15].xyz, obj_pos[i+15].w );\n"
    "\n"
    "        // calc 8-11\n"
    "        b0 = vec4( dot( v0.xyz, ray_dir ),\n"
    "                   dot( v1.xyz, ray_dir ),\n"
    "                   dot( v2.xyz, ray_dir ),\n"
    "                   dot( v3.xyz, ray_dir ) );\n"
    "\n"
    "        d2 = vec4( b0.x * b0.x - dot( v0.xyz, v0.xyz ) + v0.w,\n"
    "                   b0.y * b0.y - dot( v1.xyz, v1.xyz ) + v1.w,\n"
    "                   b0.z * b0.z - dot( v2.xyz, v2.xyz ) + v2.w,\n"
    "                   b0.w * b0.w - dot( v3.xyz, v3.xyz ) + v3.w );\n"
    "\n"
    "        // preload 16-19\n"
    "        v0 = vec4( ray_pos - obj_pos[i+16].xyz, obj_pos[i+16].w );\n"
    "        v1 = vec4( ray_pos - obj_pos[i+17].xyz, obj_pos[i+17].w );\n"
    "        v2 = vec4( ray_pos - obj_pos[i+18].xyz, obj_pos[i+18].w );\n"
    "        v3 = vec4( ray_pos - obj_pos[i+19].xyz, obj_pos[i+19].w );\n"
    "\n"
    "        // calc 12-15\n"
    "        b0 = vec4( dot( v4.xyz, ray_dir ),\n"
    "                   dot( v5.xyz, ray_dir ),\n"
    "                   dot( v6.xyz, ray_dir ),\n"
    "                   dot( v7.xyz, ray_dir ) );\n"
    "\n"
    "        d3 = vec4( b0.x * b0.x - dot( v4.xyz, v4.xyz ) + v4.w,\n"
    "                   b0.y * b0.y - dot( v5.xyz, v5.xyz ) + v5.w,\n"
    "                   b0.z * b0.z - dot( v6.xyz, v6.xyz ) + v6.w,\n"
    "                   b0.w * b0.w - dot( v7.xyz, v7.xyz ) + v7.w );\n"
    "\n"
    "        if( any( greaterThan( d0, vec4( 0.0f ) ) ) ||\n"
    "            any( greaterThan( d1, vec4( 0.0f ) ) ) ||\n"
    "            any( greaterThan( d2, vec4( 0.0f ) ) ) ||\n"
    "            any( greaterThan( d3, vec4( 0.0f ) ) ) ) break;\n"
    "\n"
    "    }\n"
    "\n"
    "    float diffuse = i == OBJNUM ? dot( plane_norm.xyz, ray_dir ) * 1.2f : 0.0f;\n"
    "    outColor = vec4( plane_col.xyz * diffuse, colscale );\n"
    "\n"
    "}\n"
};
const char *shader_noise3d = {
    "// Based on \"Noise 3D Fly Through\" by revers\n"
    "// https://www.shadertoy.com/view/4ddXW4\n"
    "// License: Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License.\n"
    "\n"
    "// The original source has been modified so it runs in 60 fps on the NVidia Jetson TX2.\n"
    "// More information about changes here: https://www.ignorantus.com/xmasdemo2017/\n"
    "\n"
    "#version 310 es\n"
    "//#pragma optionNV(unroll all)\n"
    "//#pragma optionNV(fastmath on)\n"
    "precision highp float;\n"
    "layout(location = 0) in vec2 v_texCoord;\n"
    "layout(location = 0) out vec4 outColor;\n"
    "\n"
    "#define TABLELEN 256\n"
    "#define TABLEMASK (TABLELEN-1)\n"
    "\n"
    "layout(std140,binding = 0) uniform stuff\n"
    "{\n"
    "    float width, height;\n"
    "    float frame;\n"
    "    float colscale;\n"
    "\n"
    "    vec4 noisetable[TABLELEN];\n"
    "};\n"
    "\n"
    "#define iGlobalTime (frame)\n"
    "\n"
    "const float FOV = 0.4f;\n"
    "const float MarchDumping = 0.9f;\n"
    "\n"
    "const float Far = 38.925f;\n"
    "\n"
    "const float precis = 0.03;\n"
    "\n"
    "#define MAX_STEPS 96\n"
    "\n"
    "const float CameraSpeed = 4.5099998f;\n"
    "const float TunnelSmoothFactor = 2.0f;\n"
    "const float TunnelRadius = 0.85660005f;\n"
    "const float TunnelFreqA = 0.18003f;\n"
    "const float TunnelFreqB = 0.25f;\n"
    "const float TunnelAmpA = 3.6230998f;\n"
    "const float TunnelAmpB = 2.4324f;\n"
    "const float NoiseIsoline = 0.319f;\n"
    "const float NoiseScale = 2.9980001f;\n"
    "const vec3 Color = vec3(0.085f, 0.658f, 1.0f);\n"
    "\n"
    "#define M_NONE -1.0\n"
    "#define M_NOISE 1.0\n"
    "\n"
    "\n"
    "#define PI 3.1415926525f\n"
    "\n"
    "float noise( vec3 x00 )\n"
    "{\n"
    "    ivec3 p00 = ivec3( floor( x00 ) );\n"
    "\n"
    "    vec3 f00 = fract( x00 );\n"
    "    f00 = f00 * f00 * ( 3.0 - 2.0 * f00 );\n"
    "\n"
    "    uint n00 = uint( abs( p00.x + p00.y * 157 + 113 * p00.z ) );\n"
    "    vec4 h00 = noisetable[(n00+0u)&uint(TABLEMASK)];\n"
    "    vec4 h01 = noisetable[(n00+1u)&uint(TABLEMASK)];\n"
    "\n"
    "    h00    = mix( h00,    h01,    f00.x );\n"
    "    h00.xy = mix( h00.xz, h00.yw, f00.y );\n"
    "    h00.x  = mix( h00.x,  h00.y,  f00.z );\n"
    "\n"
    "    return h00.x;\n"
    "}\n"
    "\n"
    "float fbm( vec3 p )\n"
    "{\n"
    "    float f;\n"
    "    f  = 0.5000 * noise(p); p *= 2.01;\n"
    "    f += 0.2500 * noise(p); p *= 2.02;\n"
    "    f += 0.1250 * noise(p);\n"
    "    return f;\n"
    "}\n"
    "\n"
    "\n"
    "// by iq. http://iquilezles.org/www/articles/smin/smin.htm\n"
    "float smax( float a, float b, float k )\n"
    "{\n"
    "    float h = clamp(0.5 + 0.5 * (b - a) * 0.5f, 0.0, 1.0);\n"
    "    return mix(a, b, h) + 2.0f * h * (1.0 - h);\n"
    "}\n"
    "\n"
    "// From \"Subterranean Fly-Through\" by Shane https://www.shadertoy.com/view/XlXXWj\n"
    "vec2 path( float z )\n"
    "{\n"
    "    return vec2(TunnelAmpA * sin(z * TunnelFreqA), TunnelAmpB * cos(z * TunnelFreqB));\n"
    "//    return vec2( 1.0f, 1.0f );\n"
    "}\n"
    "\n"
    "float map( vec3 p )\n"
    "{\n"
    "    float d = ( fbm( p / NoiseScale ) - NoiseIsoline ) * NoiseScale;\n"
    "\n"
    "    float d2 = length( p.xy - path( p.z ) ) - TunnelRadius;\n"
    "\n"
    "    d = smax(d, -d2, TunnelSmoothFactor);\n"
    "\n"
    "    return d;\n"
    "}\n"
    "\n"
    "float castRay( vec3 ro, vec3 rd )\n"
    "{\n"
    "    float t = 0.0f;\n"
    "\n"
    "    float md = MarchDumping;\n"
    "\n"
    "    for( int i = 0; i < MAX_STEPS; i++ ) {\n"
    "\n"
    "        float res = map( ro + rd * t );\n"
    "        if( res < precis ) break;\n"
    "\n"
    "        t += res * md;\n"
    "\n"
    "        if( i > 24 ) md *= 1.025f;\n"
    "\n"
    "        if( t > Far ) break;\n"
    "\n"
    "    }\n"
    "\n"
    "    return t;\n"
    "}\n"
    "\n"
    "vec3 calcNormal( vec3 pos )\n"
    "{\n"
    "    vec2 eps = vec2(0.001, 0.0);\n"
    "\n"
    "    vec3 nor = vec3(map(pos + eps.xyy) - map(pos - eps.xyy),\n"
    "                    map(pos + eps.yxy) - map(pos - eps.yxy),\n"
    "                    map(pos + eps.yyx) - map(pos - eps.yyx));\n"
    "\n"
    "    return normalize(nor);\n"
    "}\n"
    "\n"
    "\n"
    "vec3 render( vec3 ro, vec3 rd )\n"
    "{\n"
    "    vec3 col = vec3(0.0);\n"
    "\n"
    "    float t = castRay(ro, rd);\n"
    "\n"
    "    if( t > Far ) return col;\n"
    "\n"
    "    vec3 pos = ro + t * rd;\n"
    "    vec3 nor = calcNormal(pos);\n"
    "\n"
    "    // material\n"
    "    col = Color + sin(t * 0.8) * 0.3;\n"
    "    col += 0.3 * sin(vec3(0.15, 0.02, 0.10) * iGlobalTime * 6.0);\n"
    "\n"
    "    // lighitng\n"
    "    float occ = 1.0f;\n"
    "    vec3 lig = -rd;\n"
    "    float amb = clamp(0.5 + 0.5 * nor.y, 0.0, 1.0);\n"
    "    float dif = clamp(dot(nor, lig), 0.0, 1.0);\n"
    "\n"
    "    float fre = pow(clamp(1.0 + dot(nor, rd), 0.0, 1.0), 2.0);\n"
    "\n"
    "    vec3 ref = reflect(rd, nor);\n"
    "    float spe = pow(clamp(dot(ref, lig), 0.0, 1.0), 100.0);\n"
    "\n"
    "    vec3 brdf = vec3(0.0);\n"
    "    brdf += 1.20 * dif * vec3(1.00, 0.90, 0.60);\n"
    "    brdf += 1.20 * spe * vec3(1.00, 0.90, 0.60) * dif;\n"
    "\n"
    "    // Additional specular lighting trick,\n"
    "    // taken from \"Wet stone\" by TDM\n"
    "    // https://www.shadertoy.com/view/ldSSzV\n"
    "//    nor = normalize(nor - normalize(rd + .5)*.2);\n"
    "    nor = normalize(nor - normalize(pos) * 0.2);\n"
    "    ref = reflect(rd, nor);\n"
    "    spe = pow(clamp(dot(ref, lig), 0.0, 1.0), 100.0);\n"
    "    brdf += 2.20 * spe * vec3(1.00, 0.90, 0.60) * dif;\n"
    "\n"
    "    brdf += 0.40 * amb * vec3(0.50, 0.70, 1.00);\n"
    "    brdf += 0.40 * fre * vec3(1.00, 1.00, 1.00);\n"
    "\n"
    "    col = col * brdf;\n"
    "\n"
    "    col = col * exp(-0.005 * t * t);\n"
    "\n"
    "    return vec3(clamp(col, 0.0, 1.0));\n"
    "}\n"
    "\n"
    "mat3 rotationZ( float a )\n"
    "{\n"
    "    float sa = sin(a);\n"
    "    float ca = cos(a);\n"
    "\n"
    "    return mat3(ca, sa, 0.0, -sa, ca, 0.0, 0.0, 0.0, 1.0);\n"
    "}\n"
    "\n"
    "void main()\n"
    "{\n"
    "    vec2 coord = 2.0 * v_texCoord - 1.0;\n"
    "    coord.x *= width / height;\n"
    "    coord *= FOV;\n"
    "\n"
    "    float t = iGlobalTime * CameraSpeed + 4.0 * 60.0;\n"
    "    vec3 ro = vec3(path(t), t);\n"
    "\n"
    "    t += 0.5;\n"
    "    vec3 target = vec3(path(t), t);\n"
    "    vec3 dir = normalize(target - ro);\n"
    "    vec3 up = vec3(-0.9309864, -0.33987653, 0.1332234) * rotationZ(iGlobalTime * 0.05);\n"
    "    vec3 upOrtho = normalize(up - dot(dir, up) * dir);\n"
    "    vec3 right = normalize(cross(dir, upOrtho));\n"
    "\n"
    "    vec3 rd = normalize(dir + coord.x * right + coord.y * upOrtho);\n"
    "\n"
    "    vec3 col = render(ro, rd);\n"
    "\n"
    "    col = pow(col, vec3(0.4545));\n"
    "\n"
    "    outColor = vec4(col, colscale);\n"
    "}\n"
};
const char *shader_quat = {
    "// Based on code by Keenan Crane\n"
    "// http://www.cs.cmu.edu/~kmcrane/Projects/QuaternionJulia/\n"
    "// License: Unspecified License\n"
    "\n"
    "// The original source has been modified so it runs in 60 fps on the NVidia Jetson TX2.\n"
    "// More information about changes here: https://www.ignorantus.com/xmasdemo2017/\n"
    "\n"
    "#version 310 es\n"
    "precision highp float;\n"
    "layout(location = 0) in vec2 v_texCoord;\n"
    "layout(location = 0) out vec4 outColor;\n"
    "\n"
    "layout(std140,binding = 0) uniform stuff\n"
    "{\n"
    "    float width, height;\n"
    "    float frame;\n"
    "    float colscale;\n"
    "\n"
    "    vec4 start_pos;\n"
    "    vec4 mu_pos;\n"
    "    vec4 obj_col;\n"
    "    float rot_xz;\n"
    "    float epsilon;\n"
    "    float ax;\n"
    "    float ay;\n"
    "};\n"
    "\n"
    "#define ITERATIONS         10\n"
    "#define BOUNDING_RADIUS_2   3.0f\n"
    "#define ESCAPE_THRESHOLD   10.0f\n"
    "#define DEL                 0.0001f\n"
    "\n"
    "// Q1 Q2 = [ x1 x2 - y1 y2 - z1 z2 - w1 w2 ] + | q1.x*q2.x     - dot( q1.yzw, q2.yzw )\n"
    "//         [ x1 y2 + y1 x2 + z1 w2 - w1 z2 ] + | q1.x * q2.yzw + q2x * q1.yzw + cross( q1.yzw, q2.yzw )\n"
    "//         [ x1 z2 + z1 x2 + w1 y2 - y1 w2 ] + | ...             ...            ...\n"
    "//         [ x1 w2 + w1 x2 + y1 z2 - z1 y2 ] + | ...             ...            ...\n"
    "\n"
    "/*\n"
    "vec4 quatMult( vec4 q1, vec4 q2 )\n"
    "{\n"
    "    vec4 r;\n"
    "    r.x   = q1.x * q2.x - dot( q1.yzw, q2.yzw );\n"
    "    r.yzw = q1.x * q2.yzw + q2.x * q1.yzw + cross( q1.yzw, q2.yzw );\n"
    "    return r;\n"
    "}\n"
    "*/\n"
    "\n"
    "// don't use cross() and don't replace with dot():\n"
    "vec4 quatMult( vec4 q1, vec4 q2 )\n"
    "{\n"
    "    vec4 r;\n"
    "    r.x   = q1.x * q2.x - dot( q1.yzw, q2.yzw );\n"
    "    r.yzw = q1.x * q2.yzw + q2.x * q1.yzw  + q1.zwy * q2.wyz - q1.wyz * q2.zwy;\n"
    "    return r;\n"
    "}\n"
    "\n"
    "vec4 quatSq( vec4 q )\n"
    "{\n"
    "    vec4 r;\n"
    "    r.x   = q.x * q.x - dot( q.yzw, q.yzw );\n"
    "    r.yzw = 2.0f * q.x * q.yzw;\n"
    "    return r;\n"
    "}\n"
    "\n"
    "vec4 iterateIntersect( inout vec4 q1, vec4 c )\n"
    "{\n"
    "    bool r0,  r1;\n"
    "    vec4 q0;\n"
    "    vec4 qp0, qp1;\n"
    "\n"
    "    // reduce for qp1=vec4( 1.0f, 0.0f, 0.0f, 0.0f )\n"
    "    qp0 = 2.0f *           q1;        q0 = quatSq( q1 ) + c;\n"
    "    qp1 = 2.0f * quatMult( q0, qp0 ); q1 = quatSq( q0 ) + c;\n"
    "    r0  = dot( q0, q0 ) > ESCAPE_THRESHOLD;\n"
    "    r1  = dot( q1, q1 ) > ESCAPE_THRESHOLD;\n"
    "    if( r0 || r1 ) {\n"
    "        q1   = r0 ? q0  : q1;\n"
    "        return r0 ? qp0 : qp1;\n"
    "    }\n"
    "\n"
    "    // unroll 2 to reduce stalls\n"
    "    for( int i = 0; i < ITERATIONS-2; i += 2 ) {\n"
    "        qp0 = 2.0f * quatMult( q1, qp1 ); q0 = quatSq( q1 ) + c;\n"
    "        qp1 = 2.0f * quatMult( q0, qp0 ); q1 = quatSq( q0 ) + c;\n"
    "        r0  = dot( q0, q0 ) > ESCAPE_THRESHOLD;\n"
    "        r1  = dot( q1, q1 ) > ESCAPE_THRESHOLD;\n"
    "        if( r0 || r1 ) break;\n"
    "    }\n"
    "\n"
    "    q1   = r0 ? q0  : q1;\n"
    "    return r0 ? qp0 : qp1;\n"
    "}\n"
    "\n"
    "float intersectQJulia( inout vec3 rO, vec3 rD, vec4 c )\n"
    "{\n"
    "    float dist;\n"
    "\n"
    "    while( true ) {\n"
    "        vec4 z  = vec4( rO, 0.0f );\n"
    "        vec4 zp = iterateIntersect( z, c );\n"
    "\n"
    "        float normZ = length( z );\n"
    "        dist = 0.5f * normZ * log( normZ ) / length( zp );\n"
    "\n"
    "        rO += rD * dist;\n"
    "\n"
    "        if( dist < epsilon || dot( rO, rO ) > BOUNDING_RADIUS_2 ) break;\n"
    "    }\n"
    "\n"
    "    return dist;\n"
    "}\n"
    "\n"
    "vec3 normEstimate( vec3 qP, vec4 c )\n"
    "{\n"
    "    vec4 gx1 = vec4( qP, 0.0f ); gx1.x -= DEL;\n"
    "    vec4 gx2 = vec4( qP, 0.0f ); gx2.x += DEL;\n"
    "    vec4 gy1 = vec4( qP, 0.0f ); gy1.y -= DEL;\n"
    "    vec4 gy2 = vec4( qP, 0.0f ); gy2.y += DEL;\n"
    "    vec4 gz1 = vec4( qP, 0.0f ); gz1.z -= DEL;\n"
    "    vec4 gz2 = vec4( qP, 0.0f ); gz2.z += DEL;\n"
    "\n"
    "    // reduce iteration 0 where z=0\n"
    "    float dyz = dot( qP.yz, qP.yz );\n"
    "    float qx  = qP.x * qP.x;\n"
    "    float qx2 = 2.0f * qP.x;\n"
    "\n"
    "    gx1 = vec4( gx1.x * gx1.x - dyz,                   2.0f * gx1.x * qP.yz,  0.0f ) + c;\n"
    "    gx2 = vec4( gx2.x * gx2.x - dyz,                   2.0f * gx2.x * qP.yz,  0.0f ) + c;\n"
    "    gy1 = vec4( qx            - dot( gy1.yz, gy1.yz ), qx2          * gy1.yz, 0.0f ) + c;\n"
    "    gy2 = vec4( qx            - dot( gy2.yz, gy2.yz ), qx2          * gy2.yz, 0.0f ) + c;\n"
    "    gz1 = vec4( qx            - dot( gz1.yz, gz1.yz ), qx2          * gz1.yz, 0.0f ) + c;\n"
    "    gz2 = vec4( qx            - dot( gz2.yz, gz2.yz ), qx2          * gz2.yz, 0.0f ) + c;\n"
    "\n"
    "    for( int i = 0; i < ITERATIONS-1; i++ ) {\n"
    "        gx1 = quatSq( gx1 ) + c;\n"
    "        gx2 = quatSq( gx2 ) + c;\n"
    "        gy1 = quatSq( gy1 ) + c;\n"
    "        gy2 = quatSq( gy2 ) + c;\n"
    "        gz1 = quatSq( gz1 ) + c;\n"
    "        gz2 = quatSq( gz2 ) + c;\n"
    "    }\n"
    "\n"
    "    return normalize( vec3( length( gx2 ) - length( gx1 ),\n"
    "                            length( gy2 ) - length( gy1 ),\n"
    "                            length( gz2 ) - length( gz1 ) ) );\n"
    "}\n"
    "\n"
    "vec3 Phong( vec3 light, vec3 eye, vec3 pt, vec3 N )\n"
    "{\n"
    "    vec3  L  = normalize( light - pt );\n"
    "    vec3  E  = normalize( eye   - pt );\n"
    "    float NL = dot( N, L );\n"
    "    vec3  R  = L - 2.0f * NL * N;\n"
    "\n"
    "    float diffuse = max( NL, 0.0f );\n"
    "\n"
    "    return obj_col.xyz * diffuse + pow( max( dot( E, R ), 0.0f ), 8.0f );\n"
    "}\n"
    "\n"
    "\n"
    "#define PI 3.14159265f\n"
    "\n"
    "vec3 rotate_xz( float frac, float radius )\n"
    "{\n"
    "    return vec3(  radius*sin( frac ),\n"
    "                  0.0f,\n"
    "                 -radius*cos( frac ) );\n"
    "}\n"
    "\n"
    "vec4 getBG( void )\n"
    "{\n"
    "    vec2 df = vec2( 16.0f*3.0f, 9.0f*3.0f );\n"
    "    vec3 plane_col = obj_col.yzx;\n"
    "    vec2 pulse_pos = vec2( 0.0f );\n"
    "\n"
    "    vec2 dxy = v_texCoord - 0.5f;\n"
    "    dxy = floor( dxy * df )/df - pulse_pos/32.0f;\n"
    "    float dv = 1.0f - sqrt( dot( dxy, dxy ) + (sin(fract(frame*0.5f)*2.0f*3.14f)+1.0f)*0.5f );\n"
    "    return vec4( plane_col.yzx * dv, 1.0f );\n"
    "}\n"
    "\n"
    "void main()\n"
    "{\n"
    "    vec3 rD = normalize( vec3( (start_pos.x + ax * v_texCoord.x),\n"
    "                               (start_pos.y + ay * v_texCoord.y),\n"
    "                               (start_pos.z                    ) ) );\n"
    "\n"
    "    // was probably not a good idea to hardcode this\n"
    "\n"
    "    float angle = atan( rD.x ) * (2.0f/16.0f);\n"
    "\n"
    "    vec3 rO = rotate_xz( rot_xz,         22.0f );\n"
    "    vec3 xz = rotate_xz( rot_xz + angle,  8.0f );\n"
    "\n"
    "    rD = normalize( vec3( -xz.x, rD.y, -xz.z ) );\n"
    "\n"
    "    vec3 light = rO;\n"
    "\n"
    "    // inline intersect\n"
    "    float b0 = dot( rO, rD );\n"
    "    float d0 = b0 * b0 - dot( rO, rO ) + BOUNDING_RADIUS_2;\n"
    "\n"
    "    // redundant at correct zoom level\n"
    "    if( d0 <= 0.0f ) {\n"
    "        outColor = getBG();\n"
    "        outColor.a = colscale;\n"
    "        return;\n"
    "    }\n"
    "\n"
    "    float t0 = -b0 - sqrt( d0 );\n"
    "\n"
    "    rO += t0 * rD;\n"
    "\n"
    "    float dist = intersectQJulia( rO, rD, mu_pos );\n"
    "\n"
    "    if( dist >= epsilon ) {\n"
    "        outColor = getBG();\n"
    "        outColor.a = colscale;\n"
    "        return;\n"
    "    }\n"
    "\n"
    "    vec3 N = normEstimate( rO, mu_pos );\n"
    "\n"
    "    outColor.rgb = Phong( light, rD, rO, N );\n"
    "    outColor.a = colscale;\n"
    "\n"
    "}\n"
};
const char *shader_colorful = {
    "// Based on \"More Colorful Than Average\" by ollj\n"
    "// https://www.shadertoy.com/view/ltBcRc\n"
    "// License: Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License.\n"
    "\n"
    "// The original source has been modified so it runs in 60 fps on the NVidia Jetson TX2.\n"
    "// More information about changes here: https://www.ignorantus.com/xmasdemo2017/\n"
    "\n"
    "#version 310 es\n"
    "precision highp float;\n"
    "layout(location = 0) in vec2 v_texCoord;\n"
    "layout(location = 0) out vec4 outColor;\n"
    "\n"
    "layout(std140,binding = 0) uniform stuff\n"
    "{\n"
    "    float width, height;\n"
    "    float frame;\n"
    "    float colscale;\n"
    "};\n"
    "\n"
    "//#define iTime frame\n"
    "#define iTime tframe\n"
    "\n"
    "const vec2 shift = vec2( 0.033, 0.14);\n"
    "const vec2 shift2 = vec2( -0.023, -0.22);\n"
    "\n"
    "#define l 100 // 120\n"
    "\n"
    "void main( void )\n"
    "{\n"
    "    // timing point for music\n"
    "    float frameoff = frame-(60.0f*60.0f/60.0f);\n"
    "\n"
    "    uint no = uint( frameoff * 60.0f );\n"
    "    no = no % 3100u;\n"
    "    float tframe = float( no ) / 60.0f;\n"
    "\n"
    "\n"
    "    vec2 iResolution = vec2( width, height );\n"
    "    vec2 FragCoord = (1.0f - v_texCoord) * iResolution;\n"
    "\n"
    "    vec2 v = (FragCoord.xy - iResolution.xy/2.) / min(iResolution.y,iResolution.x) * 30.;\n"
    "    vec2 vv = v;\n"
    "//    float ft = iTime+360.1;\n"
    "    float ft = iTime*0.8f+364.1;\n"
    "    float tm = ft*0.1;\n"
    "    float tm2 = ft*0.3;\n"
    "    vec2 mspt = (vec2(  sin(tm)+cos(tm*0.2)+sin(tm*0.5)+cos(tm*-0.4)+sin(tm*1.3),\n"
    "                        cos(tm)+sin(tm*0.1)+cos(tm*0.8)+sin(tm*-1.1)+cos(tm*1.5)\n"
    "                     )+1.0)*0.35; //5x harmonics, scale back to [0,1]\n"
    "    float R = 0.0;\n"
    "    float RR = 0.0;\n"
    "    float RRR = 0.0;\n"
    "    float a = (1.-mspt.x)*0.5;\n"
    "    float C = cos(tm2*0.03+a*0.01)*1.1;\n"
    "    float S = sin(tm2*0.033+a*0.23)*1.1;\n"
    "    float C2 = cos(tm2*0.024+a*0.23)*3.1;\n"
    "    float S2 = sin(tm2*0.03+a*0.01)*3.3;\n"
    "    vec2 xa=vec2(C, -S);\n"
    "    vec2 ya=vec2(S, C);\n"
    "    vec2 xa2=vec2(C2, -S2);\n"
    "    vec2 ya2=vec2(S2, C2);\n"
    "    float Z = 0.4 + mspt.y*0.3;\n"
    "    float m = 0.99+sin(iTime*0.03)*0.003;\n"
    "\n"
    "    int i;\n"
    "    for ( i = 0; i < l; i += 2 ){\n"
    "\n"
    "        float r0  = dot(v, v);\n"
    "        float rr0 = dot(vv,vv);\n"
    "\n"
    "        if ( r0 > 1.0 )\n"
    "        {\n"
    "            r0 = 1.0f / r0;\n"
    "            v *= r0;\n"
    "        }\n"
    "        if ( rr0 > 1.0 )\n"
    "        {\n"
    "            rr0 = 1.0f / rr0;\n"
    "            vv *= rr0;\n"
    "        }\n"
    "\n"
    "        v  = vec2( dot(v,  xa),  dot(v,  ya))  * Z + shift;\n"
    "        vv = vec2( dot(vv, xa2), dot(vv, ya2)) * Z + shift2;\n"
    "\n"
    "        float r1  = dot(v, v);\n"
    "        float rr1 = dot(vv,vv);\n"
    "\n"
    "        if ( r1 > 1.0 )\n"
    "        {\n"
    "            r1 = 1.0f / r1;\n"
    "            v *= r1;\n"
    "        }\n"
    "        if ( rr1 > 1.0 )\n"
    "        {\n"
    "            rr1 = 1.0f / rr1;\n"
    "            vv *= rr1;\n"
    "        }\n"
    "\n"
    "        v  = vec2( dot(v,  xa),  dot(v,  ya))  * Z + shift;\n"
    "        vv = vec2( dot(vv, xa2), dot(vv, ya2)) * Z + shift2;\n"
    "\n"
    "        R   = ( ( R   * m + r0 ) * m ) + rr0;\n"
    "        RR  = ( ( RR  * m + r0 ) * m ) + rr0;\n"
    "        RRR = ( ( RRR * m + r0 ) * m ) + rr0;\n"
    "\n"
    "        R   = ( ( R   * m + r1 ) * m ) + rr1;\n"
    "        RR  = ( ( RR  * m + r1 ) * m ) + rr1;\n"
    "        RRR = ( ( RRR * m + r1 ) * m ) + rr1;\n"
    "\n"
    "    }\n"
    "\n"
    "    // l-2\n"
    "    float r  = dot(v, v);\n"
    "    float rr = dot(vv,vv);\n"
    "\n"
    "    if ( r > 1.0 )\n"
    "    {\n"
    "            r  = 1.0f / r;\n"
    "            v *= r;\n"
    "    }\n"
    "    if ( rr > 1.0 )\n"
    "    {\n"
    "        rr  = 1.0f / rr;\n"
    "        vv *= rr;\n"
    "    }\n"
    "\n"
    "    v  = vec2( dot(v,  xa),  dot(v,  ya))  * Z + shift;\n"
    "    vv = vec2( dot(vv, xa2), dot(vv, ya2)) * Z + shift2;\n"
    "\n"
    "    R   = ( ( R   * m + r ) * m ) + rr;\n"
    "    RR  = ( ( RR  * m + r ) * m ) + rr;\n"
    "\n"
    "\n"
    "    // l-1\n"
    "    r  = dot(v, v);\n"
    "    rr = dot(vv,vv);\n"
    "\n"
    "    if ( r > 1.0 )\n"
    "    {\n"
    "            r  = 1.0f / r;\n"
    "    }\n"
    "    if ( rr > 1.0 )\n"
    "    {\n"
    "        rr  = 1.0f / rr;\n"
    "    }\n"
    "\n"
    "    R   = ( ( R   * m + r ) * m ) + rr;\n"
    "\n"
    "/*\n"
    "    for ( ; i < l; i++ ){\n"
    "        float r = dot(v,v);\n"
    "        float r2 = dot(vv,vv);\n"
    "        if ( r > 1.0 )\n"
    "        {\n"
    "            r = (1.0)/r ;\n"
    "            v.x = v.x * r;\n"
    "            v.y = v.y * r;\n"
    "        }\n"
    "        if ( r2 > 1.0 )\n"
    "        {\n"
    "            r2 = (1.0)/r2 ;\n"
    "            vv.x = vv.x * r2;\n"
    "            vv.y = vv.y * r2;\n"
    "        }\n"
    "        R *= m;\n"
    "        R += r;\n"
    "        R *= m;\n"
    "        R += r2;\n"
    "        if(i < l-1){\n"
    "            RR *= m;\n"
    "            RR += r;\n"
    "            RR *= m;\n"
    "            RR += r2;\n"
    "            if(i < l-2){\n"
    "                RRR *= m;\n"
    "                RRR += r;\n"
    "                RRR *= m;\n"
    "                RRR += r2;\n"
    "            }\n"
    "        }\n"
    "\n"
    "        v = vec2( dot(v, xa), dot(v, ya)) * Z + shift;\n"
    "        vv = vec2( dot(vv, xa2), dot(vv, ya2)) * Z + shift2;\n"
    "    }\n"
    "*/\n"
    "\n"
    "//  float c = ((mod(R,2.0)>1.0)?1.0-fract(R):fract(R));\n"
    "//  float cc = ((mod(RR,2.0)>1.0)?1.0-fract(RR):fract(RR));\n"
    "//  float ccc = ((mod(RRR,2.0)>1.0)?1.0-fract(RRR):fract(RRR));\n"
    "\n"
    "    float rf0 = fract( R );\n"
    "    float rf1 = fract( RR );\n"
    "    float rf2 = fract( RRR );\n"
    "    float c   = ((mod(R,  2.0)>1.0) ? 1.0-rf0 : rf0);\n"
    "    float cc  = ((mod(RR, 2.0)>1.0) ? 1.0-rf1 : rf1);\n"
    "    float ccc = ((mod(RRR,2.0)>1.0) ? 1.0-rf2 : rf2);\n"
    "\n"
    "\n"
    "//    float c   = ((int(R)  &1) == 1) ? 1.0-fract(R)  : fract(R);\n"
    "//    float cc  = ((int(RR) &1) == 1) ? 1.0-fract(RR) : fract(RR);\n"
    "//    float ccc = ((int(RRR)&1) == 1) ? 1.0-fract(RRR): fract(RRR);\n"
    "\n"
    "    outColor = vec4( vec3( ccc, cc, c ), colscale );\n"
    "}\n"
};
const char *shader_seascape = {
    "// Based on \"Seascape\" by TDM\n"
    "// https://www.shadertoy.com/view/Ms2SD1\n"
    "// License: Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License.\n"
    "\n"
    "// The original source has been modified so it runs in 60 fps on the NVidia Jetson TX2.\n"
    "// More information about changes here: https://www.ignorantus.com/xmasdemo2017/\n"
    "\n"
    "#version 310 es\n"
    "precision highp float;\n"
    "layout(location = 0) in vec2 v_texCoord;\n"
    "layout(location = 0) out vec4 outColor;\n"
    "\n"
    "layout(std140,binding = 0) uniform stuff\n"
    "{\n"
    "    float width, height;\n"
    "    float frame;\n"
    "    float colscale;\n"
    "    vec4 ang;\n"
    "    vec4 ori;\n"
    "    vec4 m0, m1, m2;\n"
    "};\n"
    "\n"
    "const int NUM_STEPS = 8;\n"
    "const float PI      = 3.141592;\n"
    "const float EPSILON = 1e-3;\n"
    "#define EPSILON_NRM (0.1 / iResolution.x)\n"
    "\n"
    "#define iTime (frame)\n"
    "\n"
    "// sea\n"
    "const int ITER_GEOMETRY = 3;\n"
    "const int ITER_FRAGMENT = 5;\n"
    "const float SEA_HEIGHT = 0.6;\n"
    "const float SEA_CHOPPY = 4.0;\n"
    "const float SEA_SPEED = 0.8;\n"
    "const float SEA_FREQ = 0.16;\n"
    "const vec3 SEA_BASE = vec3(0.1,0.19,0.22);\n"
    "const vec3 SEA_WATER_COLOR = vec3(0.8,0.9,0.6);\n"
    "#define SEA_TIME (1.0 + iTime * SEA_SPEED)\n"
    "const mat2 octave_m = mat2(1.6,1.2,-1.2,1.6);\n"
    "\n"
    "float noise( in vec2 p )\n"
    "{\n"
    "    float f = ( sin( p.x * 1.91f ) + cos( p.y * 1.71f ) ) * 0.75f;\n"
    "    return f;\n"
    "}\n"
    "\n"
    "\n"
    "\n"
    "// lighting\n"
    "float diffuse( vec3 n, vec3 l, float p )\n"
    "{\n"
    "    return pow(dot(n,l) * 0.4 + 0.6,p);\n"
    "}\n"
    "\n"
    "float specular( vec3 n, vec3 l, vec3 e, float s )\n"
    "{\n"
    "    float nrm = (s + 8.0) / (PI * 8.0);\n"
    "    return pow(max(dot(reflect(e,n),l),0.0),s) * nrm;\n"
    "}\n"
    "\n"
    "// sky\n"
    "vec3 getSkyColor( vec3 e )\n"
    "{\n"
    "    e.y = max(e.y,0.0);\n"
    "    return vec3(pow(1.0-e.y,2.0), 1.0-e.y, 0.6+(1.0-e.y)*0.4);\n"
    "}\n"
    "\n"
    "// sea\n"
    "float sea_octave( vec2 uv, float choppy )\n"
    "{\n"
    "    uv += noise(uv);\n"
    "    vec2 wv = 1.0-abs(sin(uv));\n"
    "    vec2 swv = abs(cos(uv));\n"
    "    wv = mix(wv,swv,wv);\n"
    "    return pow(1.0-pow(wv.x * wv.y,0.65),choppy);\n"
    "}\n"
    "\n"
    "float map( vec3 p )\n"
    "{\n"
    "    float freq = SEA_FREQ;\n"
    "    float amp = SEA_HEIGHT;\n"
    "    float choppy = SEA_CHOPPY;\n"
    "    vec2 uv = p.xz; uv.x *= 0.75;\n"
    "\n"
    "    float d, h = 0.0;\n"
    "    for(int i = 0; i < ITER_GEOMETRY; i++) {\n"
    "        d = sea_octave((uv+SEA_TIME)*freq,choppy) * 1.4f;\n"
    "        h += d * amp;\n"
    "        uv *= octave_m; freq *= 1.9; amp *= 0.22;\n"
    "        choppy = mix(choppy,1.0,0.2);\n"
    "    }\n"
    "    return p.y - h;\n"
    "}\n"
    "\n"
    "float map_detailed( vec3 p )\n"
    "{\n"
    "    float freq = SEA_FREQ;\n"
    "    float amp = SEA_HEIGHT;\n"
    "    float choppy = SEA_CHOPPY;\n"
    "    vec2 uv = p.xz; uv.x *= 0.75;\n"
    "\n"
    "    float d, h = 0.0;\n"
    "    for(int i = 0; i < ITER_FRAGMENT; i++) {\n"
    "        d = sea_octave((uv+SEA_TIME)*freq,choppy);\n"
    "        d += sea_octave((uv-SEA_TIME)*freq,choppy);\n"
    "        h += d * amp;\n"
    "        uv *= octave_m; freq *= 1.9; amp *= 0.22;\n"
    "        choppy = mix(choppy,1.0,0.2);\n"
    "    }\n"
    "    return p.y - h;\n"
    "}\n"
    "\n"
    "vec3 getSeaColor( vec3 p, vec3 n, vec3 l, vec3 eye, vec3 dist )\n"
    "{\n"
    "    float fresnel = clamp(1.0 - dot(n,-eye), 0.0, 1.0);\n"
    "    fresnel = pow(fresnel,3.0) * 0.65;\n"
    "\n"
    "    vec3 reflected = getSkyColor(reflect(eye,n));\n"
    "    vec3 refracted = SEA_BASE + diffuse(n,l,80.0) * SEA_WATER_COLOR * 0.12;\n"
    "\n"
    "    vec3 color = mix(refracted,reflected,fresnel);\n"
    "\n"
    "    float atten = max(1.0 - dot(dist,dist) * 0.001, 0.0);\n"
    "    color += SEA_WATER_COLOR * (p.y - SEA_HEIGHT) * 0.18 * atten;\n"
    "\n"
    "    color += vec3(specular(n,l,eye,60.0));\n"
    "\n"
    "    return color;\n"
    "}\n"
    "\n"
    "// tracing\n"
    "vec3 getNormal( vec3 p, float eps )\n"
    "{\n"
    "    vec3 n;\n"
    "    n.y = map_detailed(p);\n"
    "    n.x = map_detailed(vec3(p.x+eps,p.y,p.z)) - n.y;\n"
    "    n.z = map_detailed(vec3(p.x,p.y,p.z+eps)) - n.y;\n"
    "    n.y = eps;\n"
    "    return normalize(n);\n"
    "}\n"
    "\n"
    "vec3 heightMapTracing( vec3 ori, vec3 dir, out float hx )\n"
    "{\n"
    "    vec3 p;\n"
    "    float tm = 0.0;\n"
    "    float tx = 1000.0;\n"
    "//    float hm = map(ori);\n"
    "    float hm = 1.0f; // ori fixed per frame, so close enough\n"
    "    float tmid = 0.0;\n"
    "\n"
    "    for(int i = 0; i < NUM_STEPS; i++) {\n"
    "        tmid = mix(tm,tx, hm/(hm-hx));\n"
    "        p = ori + dir * tmid;\n"
    "        float hmid = map(p);\n"
    "        if(hmid < 0.0) {\n"
    "            tx = tmid;\n"
    "            hx = hmid;\n"
    "        } else {\n"
    "            tm = tmid;\n"
    "            hm = hmid;\n"
    "        }\n"
    "    }\n"
    "    return p;\n"
    "}\n"
    "\n"
    "void main( void )\n"
    "{\n"
    "    vec2 iResolution = vec2( width, height );\n"
    "\n"
    "    vec2 fragCoord = (1.0f - v_texCoord) * iResolution;\n"
    "\n"
    "    vec2 uv = fragCoord.xy / iResolution.xy;\n"
    "    uv = uv * 2.0 - 1.0;\n"
    "    uv.x *= iResolution.x / iResolution.y;\n"
    "    float time = iTime * 0.3;\n"
    "\n"
    "    // ray\n"
    "    mat3 m = mat3( m0.xyz, m1.xyz, m2.xyz );\n"
    "\n"
    "    vec3 dir = normalize(vec3(uv.xy,-2.0)) * m;\n"
    "\n"
    "    // tracing\n"
    "    float hx = map( ori.xyz + dir * 1000.0f );\n"
    "    if( hx > 0.0f ) {\n"
    "        outColor = vec4( getSkyColor(dir), colscale );\n"
    "        return;\n"
    "    }\n"
    "\n"
    "    vec3 p = heightMapTracing(ori.xyz,dir, hx );\n"
    "\n"
    "    vec3 dist = p - ori.xyz;\n"
    "    vec3 n = getNormal(p, dot(dist,dist) * EPSILON_NRM);\n"
    "\n"
    "    vec3 light = vec3( 0.000000f, 0.780869f, 0.624695f );\n"
    "\n"
    "    // color\n"
    "    vec3 color = mix(\n"
    "        getSkyColor(dir),\n"
    "        getSeaColor(p,n,light,dir,dist),\n"
    "        pow(smoothstep(0.0,-0.05,dir.y),0.3));\n"
    "\n"
    "    // post\n"
    "    outColor = vec4( pow( color, vec3( 0.75 ) ), colscale );\n"
    "}\n"
};
const char *shader_trans = {
    "// Based on \"Transparent Lattice\" by Shane\n"
    "// https://www.shadertoy.com/view/Xd3SDs\n"
    "// License: Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License.\n"
    "\n"
    "// The original source has been modified so it runs in 60 fps on the NVidia Jetson TX2.\n"
    "// More information about changes here: https://www.ignorantus.com/xmasdemo2017/\n"
    "\n"
    "#version 310 es\n"
    "precision highp float;\n"
    "layout(location = 0) in vec2 v_texCoord;\n"
    "layout(location = 0) out vec4 outColor;\n"
    "\n"
    "layout(std140,binding = 0) uniform stuff\n"
    "{\n"
    "    float width, height;\n"
    "    float frame;\n"
    "    float colscale;\n"
    "};\n"
    "\n"
    "#define iTime (frame)\n"
    "\n"
    "\n"
    "// Cheap vec3 to vec3 hash. Works well enough, but there are other ways.\n"
    "vec3 hash33( vec3 p )\n"
    "{\n"
    "    float n = sin(dot(p, vec3(7, 157, 113)));\n"
    "    return fract(vec3(2097152, 262144, 32768)*n);\n"
    "}\n"
    "\n"
    "\n"
    "// A fake noise looking field. Pretty interesting.\n"
    "\n"
    "float map( vec3 p )\n"
    "{\n"
    "    p = (cos(p*.315*2.5 + sin(p.zxy*.875*2.5)));\n"
    "\n"
    "    float n = length(p);\n"
    "\n"
    "    p = sin(p*6. + cos(p.yzx*6.));\n"
    "\n"
    "    return n - 1. - abs(p.x*p.y*p.z)*.05;\n"
    "}\n"
    "\n"
    "\n"
    "// Not big on accuracy, but lower on operations. Few distance function calls are important\n"
    "// during volumetric passes.\n"
    "/*\n"
    "vec3 calcNormal( vec3 p, float d )\n"
    "{\n"
    "    const vec2 e = vec2(0.01, 0);\n"
    "    return normalize(vec3(d - map(p - e.xyy), d - map(p - e.yxy), d - map(p - e.yyx)));\n"
    "}\n"
    "*/\n"
    "\n"
    "vec3 calcNormal( vec3 p, float d )\n"
    "{\n"
    "    vec3 r0 = cos( p * .315 * 2.5 + sin( p.zxy * .875 * 2.5 ) ); p -= 0.01f;\n"
    "    vec3 r1 = cos( p * .315 * 2.5 + sin( p.zxy * .875 * 2.5 ) );\n"
    "\n"
    "    vec3 v0 = vec3( r1.x, r0.y, r0.z );\n"
    "    vec3 v1 = vec3( r0.x, r1.y, r0.z );\n"
    "    vec3 v2 = vec3( r0.x, r0.y, r1.z );\n"
    "\n"
    "    float n0 = length( v0 );\n"
    "    float n1 = length( v1 );\n"
    "    float n2 = length( v2 );\n"
    "\n"
    "    v0 = sin( v0 * 6. + cos( v0.yzx * 6. ) );\n"
    "    v1 = sin( v0 * 6. + cos( v0.yzx * 6. ) );\n"
    "\n"
    "    float f = - 1.0f - abs( v1.x * v1.y * v1.z ) * .05;\n"
    "\n"
    "    n0 = n0 - 1.0f - abs( v0.x * v0.y * v0.z ) * .05;\n"
    "    n1 = n1 + f;\n"
    "    n2 = n2 + f;\n"
    "\n"
    "    return normalize( vec3( d - n0, d - n1, d - n2 ) );\n"
    "}\n"
    "\n"
    "/*\n"
    "// Tetrahedral normal, to save a couple of \"map\" calls. Courtesy of IQ. Unfortunately, still\n"
    "// not fast enough in this particular instance.\n"
    "vec3 calcNormal(in vec3 p){\n"
    "\n"
    "    // Note the slightly increased sampling distance, to alleviate artifacts due to hit point inaccuracies.\n"
    "    vec2 e = vec2(0.0025, -0.0025);\n"
    "    return normalize(e.xyy * map(p + e.xyy) + e.yyx * map(p + e.yyx) + e.yxy * map(p + e.yxy) + e.x * map(p + e.xxx));\n"
    "}\n"
    "*/\n"
    "\n"
    "\n"
    "void main( void )\n"
    "{\n"
    "    vec2 iResolution = vec2( width, height );\n"
    "\n"
    "    vec2 fragCoord = (1.0f - v_texCoord) * iResolution;\n"
    "\n"
    "    // Screen coordinates.\n"
    "    vec2 uv = (fragCoord.xy - iResolution.xy*.5 )/iResolution.y;\n"
    "\n"
    "    // Unit direction ray. The last term is one of many ways to fish-lens the camera.\n"
    "    // For a regular view, set \"rd.z\" to something like \"0.5.\"\n"
    "    vec3 rd = normalize(vec3(uv, (1.-dot(uv, uv)*.5)*.5)); // Fish lens, for that 1337, but tryhardish, demo look. :)\n"
    "\n"
    "    // There are a few ways to hide artifacts and inconsistencies. Making things go fast is one of them. :)\n"
    "    // Ray origin, scene color, and surface postion vector.\n"
    "    vec3 ro = vec3(0., 0., iTime*1.5);\n"
    "    vec3 col=vec3(0);\n"
    "    vec3 sp, sn, lp, ld, rnd;\n"
    "\n"
    "    // Swivel the unit ray to look around the scene.\n"
    "    // Compact 2D rotation matrix, courtesy of Shadertoy user, \"Fabrice Neyret.\"\n"
    "    vec2 a = sin(vec2(1.5707963, 0) + iTime*0.375);\n"
    "    rd.xz = mat2(a, -a.y, a.x)*rd.xz;\n"
    "    rd.xy = mat2(a, -a.y, a.x)*rd.xy;\n"
    "\n"
    "    lp = vec3(0, 1, 4);\n"
    "    lp.xz = mat2(a, -a.y, a.x)*lp.xz;\n"
    "    lp.xy = mat2(a, -a.y, a.x)*lp.xy;\n"
    "    lp += ro;\n"
    "\n"
    "    // Unit ray jitter is another way to hide artifacts. It can also trick the viewer into believing\n"
    "    // something hard core, like global illumination, is happening. :)\n"
    "    //rd *= 0.99 + hash33(rd)*0.02;\n"
    "\n"
    "    // Some more randomization, to be used for color based jittering inside the loop.\n"
    "    rnd = hash33(rd+311.);\n"
    "\n"
    "    // Ray distance, bail out layer number, surface distance and normalized accumulated distance.\n"
    "    // Note the slight jittering to begin with. It alleviates the subtle banding.\n"
    "    float t = length(rnd)*.2, d, aD;\n"
    "    int layers = 0;\n"
    "\n"
    "    // Light variables.\n"
    "    float lDist, s, l;\n"
    "\n"
    "    // Surface distance threshold. Smaller numbers gives a thinner membrane, but lessens detail...\n"
    "    // hard to explain. It's easier to check it out for yourself.\n"
    "    float thD = .0125; // + smoothstep(-0.2, 0.2, sin(iTime*0.75 - 3.14159*0.4))*0.025;\n"
    "\n"
    "    // Only a few iterations seemed to be enough. Obviously, more looks better, but is slower.\n"
    "    for(int i=0; i<40; i++) {\n"
    "\n"
    "        // Break conditions. Anything that can help you bail early usually increases frame rate.\n"
    "        if(layers>31 || t>16. || dot(col, vec3(.299, .587, .114)) > 1. ) break; // 31\n"
    "\n"
    "        // Current ray postion. Slightly redundant here, but sometimes you may wish to reuse\n"
    "        // it during the accumulation stage.\n"
    "        sp = ro+rd*t;\n"
    "\n"
    "        d = map(sp); // Distance to nearest point on the noise surface.\n"
    "\n"
    "        // If we get within a certain distance of the surface, accumulate some surface values.\n"
    "        // Values further away have less influence on the total.\n"
    "        //\n"
    "        // aD - Accumulated distance. You could smoothly interpolate it, if you wanted.\n"
    "        //\n"
    "        // 1/.(1. + t*t*0.1) - Basic distance attenuation. Feel free to substitute your own.\n"
    "\n"
    "        // Normalized distance from the surface threshold value to our current isosurface value.\n"
    "        aD = (thD-abs(d)*31./32.)/thD;\n"
    "\n"
    "        // If we're within the surface threshold, accumulate some color.\n"
    "        // Two \"if\" statements in a shader loop makes me nervous. I don't suspect there'll be any\n"
    "        // problems, but if there are, let us know.\n"
    "        if(aD>0.0) // 0.8 -> 40\n"
    "        {\n"
    "\n"
    "            // Add the accumulated surface distance value, along with some basic falloff using the\n"
    "            // camera to light distance, \"lDist.\" There's a bit of color jitter there, too.\n"
    "\n"
    "            sn = calcNormal(sp, d)*sign(d);\n"
    "//            sn = calcNormal( sp, d );\n"
    "//            uvec3 intsn = floatBitsToUint( sn );\n"
    "//            uvec3 signy = (intsn ^ floatBitsToUint( d  ))& 0x80000000u;\n"
    "//            sn = uintBitsToFloat((intsn&0x7fffffffu) | signy);\n"
    "\n"
    "\n"
    "            ld = (lp - sp); //vec3(.5773)\n"
    "            lDist = max(length(ld), .001);\n"
    "//            lDist = length(ld);\n"
    "            ld /= lDist;\n"
    "            s = pow(max(dot(reflect(-ld, sn), -rd), 0.), 8.);\n"
    "            l = max(dot(ld, sn), 0.);\n"
    "\n"
    "            //float c = dot(sin(sp*128. - cos(sp.yzx*64.)), vec3(.166))+.5;\n"
    "            col += ((l + .1) + vec3(.5, .7, 1)*s)*aD/(1. + lDist*0.25 + lDist*lDist*0.05)*.2;\n"
    "            // Failed experiment with color jitter to take out more banding.\n"
    "            //col += ((l + .05 + fract(rnd + i*27.)*.1) + vec3(.5, .7, 1)*s)*aD/(1. + lDist*0.25 + lDist*lDist*0.05)*.2;\n"
    "\n"
    "            // The layer number is worth noting. Accumulating more layers gives a bit more glow.\n"
    "            // Lower layer numbers allow a quicker bailout. A lot of it is guess work.\n"
    "            layers++;\n"
    "\n"
    "        }\n"
    "\n"
    "        // Kind of weird the way this works. I think not allowing the ray to hone in properly is\n"
    "        // the very thing that gives an even spread of values. The figures are based on a bit\n"
    "        // of knowledge versus trial and error. If you have a faster computer, feel free to tweak\n"
    "        // them a bit.\n"
    "        t += max(abs(d)*.75, thD*.25);\n"
    "\n"
    "    }\n"
    "\n"
    "    t = min(t, 16.);\n"
    "\n"
    "    col = mix(col, vec3(0), 1.-exp(-0.025*t*t));////1.-exp(-0.01*t*t) 1.-1./(1. + t*t*.1)\n"
    "\n"
    "    // Mixing the greytone color with a firey orange vignette. There's no meaning\n"
    "    // behind it. I just thought the artsy greyscale was a little too artsy.\n"
    "    uv = abs(fragCoord.xy/iResolution.xy - .5); // Wasteful, but the GPU can handle it.\n"
    "    col = mix(col, vec3(min(col.x*1.5, 1.), pow(col.x, 2.5), pow(col.x, 12.)).yxz,\n"
    "               min( dot(pow(uv, vec2(4.)), vec2(1))*8., 1.));\n"
    "\n"
    "    //col = vec3(min(col.z*1.5, 1.), pow(col.z, 2.5), pow(col.z, 12.));\n"
    "\n"
    "    // Mixing the vignette colors up a bit more.\n"
    "    col = mix(col, col.zxy, dot(sin(rd*5.), vec3(.166)) + 0.166);\n"
    "\n"
    "    // Presenting the color to the screen.\n"
    "//    outColor = vec4( sqrt(clamp(col*2.0f, 0., 1.)), 1.0 );\n"
    "    outColor = vec4( sqrt( col * 1.8f ), colscale );\n"
    "}\n"
};
const char *shader_twofield = {
    "// Based on \"Twofield\" by w23\n"
    "// https://www.shadertoy.com/view/Xs2XDV\n"
    "// License: Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License.\n"
    "\n"
    "// The original source has been modified so it runs in 60 fps on the NVidia Jetson TX2.\n"
    "// More information about changes here: https://www.ignorantus.com/xmasdemo2017/\n"
    "\n"
    "#version 310 es\n"
    "//#pragma optionNV(unroll all)\n"
    "//#pragma optionNV(fastmath on)\n"
    "precision highp float;\n"
    "layout(location = 0) in vec2 v_texCoord;\n"
    "layout(location = 0) out vec4 outColor;\n"
    "\n"
    "#define TABLELEN 256\n"
    "#define N 8\n"
    "\n"
    "layout(std140,binding = 0) uniform stuff\n"
    "{\n"
    "    float width, height;\n"
    "    float frame;\n"
    "    float colscale;\n"
    "\n"
    "    vec4 l1pos, l2pos, l3pos, l4pos;\n"
    "    vec4 l1color, l2color, l3color, l4color;\n"
    "    vec4 b1[N];\n"
    "    vec4 b2[N];\n"
    "    vec4 O;\n"
    "};\n"
    "\n"
    "#define Lmin (12.5f)\n"
    "#define Lmax (40.0f)\n"
    "#define STEPS 40\n"
    "\n"
    "#define iTime frame\n"
    "\n"
    "#define t (iTime * .3)\n"
    "\n"
    "// Simplest Lambert+Phong material shading model parameters\n"
    "struct mat_t {\n"
    "    vec3 diffuse;\n"
    "    vec4 specular;\n"
    "};\n"
    "\n"
    "// There are two separate distance fields, each consists of 8 metaballs and a plane.\n"
    "\n"
    "vec2 t1t2( vec3 p )\n"
    "{\n"
    "    float v1 = 0.0f;\n"
    "    float v2 = 0.0f;\n"
    "\n"
    "    for( int i = 0; i < N; ++i ) {\n"
    "        vec3 b1 = p - b1[i].xyz;\n"
    "        vec3 b2 = p - b2[i].xyz;\n"
    "\n"
    "        v1 += 5. / dot(b1, b1);\n"
    "        v2 += 5. / dot(b2, b2);\n"
    "\n"
    "    }\n"
    "\n"
    "    float d1 = 12. - p.y; v1 += 3. / (d1*d1);\n"
    "    float d2 = 12. + p.y; v2 += 3. / (d2*d2);\n"
    "\n"
    "    return vec2( v1, v2 );\n"
    "}\n"
    "\n"
    "\n"
    "float world( vec3 p )\n"
    "{\n"
    "    vec2 t12 = t1t2( p );\n"
    "\n"
    "    float w1 = 1.0f - t12.x + t12.y;\n"
    "    float w2 = 1.0f + t12.x - t12.y;\n"
    "\n"
    "    return min( w1, w2 );\n"
    "}\n"
    "\n"
    "// Material is picked based on which field's distance function is the smallest\n"
    "mat_t material( vec3 p )\n"
    "{\n"
    "    mat_t m;\n"
    "\n"
    "    vec2 t12 = t1t2( p );\n"
    "\n"
    "    float w1 = 1.0f - t12.x + t12.y;\n"
    "    float w2 = 1.0f + t12.x - t12.y;\n"
    "\n"
    "    if (w1 < w2) {\n"
    "        // red\n"
    "        m.diffuse  = vec3( 0.92f, 0.027f, 0.027f );\n"
    "        m.specular = vec4( 0.9440f, 0.3189f, 0.3189f, 1000.0f );\n"
    "    } else {\n"
    "        // blue\n"
    "        m.diffuse  = vec3( 0.14f, 0.17f, 1.57f );\n"
    "        m.specular = vec4( 0.5699f, 0.5850f, 1.2850f, 100.0f );\n"
    "    }\n"
    "\n"
    "    return m;\n"
    "}\n"
    "\n"
    "\n"
    "vec3 normal( vec3 p )\n"
    "{\n"
    "    vec2 e = vec2(.001,0.);\n"
    "\n"
    "    return normalize( vec3( world( p + e.xyy ) - world( p - e.xyy ),\n"
    "                            world( p + e.yxy ) - world( p - e.yxy ),\n"
    "                            world( p + e.yyx ) - world( p - e.yyx ) ) );\n"
    "}\n"
    "\n"
    "// Raymarcher\n"
    "float trace( vec3 D )\n"
    "{\n"
    "    // The usual \"step along the ray\" loop\n"
    "    float Lp;\n"
    "    float L = Lmin;\n"
    "\n"
    "    for( int i = 0; i < STEPS; i += 2 ) {\n"
    "\n"
    "        float d;\n"
    "        d = world( O.xyz + D * L ); if( d < 0.01 * L || L >= Lmax ) break; Lp = L; L += d * 1.5;\n"
    "        d = world( O.xyz + D * L ); if( d < 0.01 * L || L >= Lmax ) break; Lp = L; L += d * 1.5;\n"
    "\n"
    "    }\n"
    "\n"
    "    // Binary search for more exact intersection position\n"
    "    // Needed to fix artifacts due to overstep\n"
    "    if (L < Lmax ) {\n"
    "\n"
    "        for( int i = 0; i < 5; i++ ) {\n"
    "            float Lm = (Lp + L) * .5;\n"
    "            if( world(O.xyz + D * Lm) < .01*Lm ) L = Lm; else Lp = Lm;\n"
    "        }\n"
    "\n"
    "    }\n"
    "\n"
    "    return L;\n"
    "}\n"
    "\n"
    "\n"
    "// Compute shading color from a single light\n"
    "vec3 enlight( vec3 p, vec3 v, vec3 n, mat_t m, vec3 lpos, vec3 lcolor )\n"
    "{\n"
    "    vec3 ldir = lpos - p; // direction from the point to the light source\n"
    "    float ldist2 = dot(ldir, ldir); // square distance to the light source\n"
    "    float ldist = sqrt(ldist2); // distance\n"
    "    ldir /= ldist; // ldir is now normalized\n"
    "//    float shadow = shadowtrace(p, ldir, ldist-.2); // whether the point is in shadow\n"
    "    float shadow = 1.0f;\n"
    "    return shadow * lcolor * (\n"
    "        // Diffuse term (simple Lambert)\n"
    "        m.diffuse * max(0., dot(n,ldir))\n"
    "        // Specular term (Phong-ish)\n"
    "      + m.specular.rgb * pow(max(0.,dot(normalize(ldir-v), n)), m.specular.w)\n"
    "        // Specular energy conservation? (or something, it was 3 years ago :D)\n"
    "        * (m.specular.w + 2.) * (m.specular.w + 4.) / (24. * (m.specular.w + pow(2., -m.specular.w/2.)))\n"
    "    ) / ldist2;\n"
    "}\n"
    "\n"
    "// A ridiculous hack to display light sources\n"
    "// Basic idea is to check whether the ray is \"close enough\" to the light source position in screen space\n"
    "// Which is done by calculating the angle between the actual pixel ray and a vector from origin to the light source\n"
    "\n"
    "vec3 lightball( vec3 lpos, vec3 lcolor, vec3 O, vec3 D, float L )\n"
    "{\n"
    "    vec3 ldir = lpos - O;\n"
    "\n"
    "    if( dot( ldir, ldir ) > L*L ) return vec3( 0.0f );\n"
    "\n"
    "    float lv = 2.07f - ( ( ( lpos.z / 10.0f + 1.0f ) / 2.0f ) + 1.0f );\n"
    "\n"
    "    float pw = pow( max( 0.0f, dot( normalize( ldir ), D ) ), 20000.0f * lv );\n"
    "\n"
    "    return ( normalize( lcolor ) + 1.0f ) * pw;\n"
    "}\n"
    "\n"
    "\n"
    "#define vt v_texCoord\n"
    "\n"
    "void main( void )\n"
    "{\n"
    "    float xedge  = 256.0f/1920.0f;\n"
    "    float yedge2 = 208.0f/1080.0f;\n"
    "    float xedge2 = 416.0f/1920.0f;\n"
    "\n"
    "    if(  (vt.x < xedge  || vt.x > 1.0f - xedge) ||\n"
    "        ((vt.x < xedge2 || vt.x > 1.0f - xedge2) && vt.y > yedge2 && vt.y < 1.0f - yedge2 ) ) {\n"
    "        outColor = vec4( 0.0f, 0.0f, 0.0f, colscale );\n"
    "        return;\n"
    "    }\n"
    "\n"
    "    vec2 uv = v_texCoord * 2. - 1.;\n"
    "    uv.x *= width / height;\n"
    "\n"
    "    vec3 D = normalize(vec3(uv,-1.6));\n"
    "\n"
    "//    D = normalize(mat3(rg.xyz, rgxfw.xyz, -fw.xyz) * D);\n"
    "\n"
    "    // Raymarch\n"
    "    float L = trace( D );\n"
    "\n"
    "    vec3 color = vec3( 0.0f );\n"
    "\n"
    "    if( L < Lmax ) {\n"
    "        // If a hit is detected, then get the position\n"
    "        vec3 p = O.xyz + D * L;\n"
    "        // And normal\n"
    "        vec3 n = normal(p);\n"
    "        // And material\n"
    "        mat_t m = material(p);\n"
    "\n"
    "        // Ambient term (I don't think it is visible much)\n"
    "        color = .001 * m.diffuse * n;\n"
    "\n"
    "        // Shade for all lights\n"
    "        color += enlight(p, D, n, m, l1pos.xyz, l1color.xyz);\n"
    "        color += enlight(p, D, n, m, l2pos.xyz, l2color.xyz);\n"
    "        color += enlight(p, D, n, m, l3pos.xyz, l3color.xyz);\n"
    "        color += enlight(p, D, n, m, l4pos.xyz, l4color.xyz);\n"
    "\n"
    "        // Fog-like hack-ish attenuation, so the only the center of the scene is visible\n"
    "        color *= (1. - smoothstep(12., 18., length(p)));\n"
    "\n"
    "    } else L = 100.0f;\n"
    "\n"
    "    // Add light sources hack\n"
    "    color += lightball(l1pos.xyz, l1color.xyz, O.xyz, D, L );\n"
    "    color += lightball(l2pos.xyz, l2color.xyz, O.xyz, D, L );\n"
    "    color += lightball(l3pos.xyz, l3color.xyz, O.xyz, D, L );\n"
    "    color += lightball(l4pos.xyz, l4color.xyz, O.xyz, D, L );\n"
    "\n"
    "    outColor = vec4( color,colscale );\n"
    "}\n"
};
const char *shader_torus = {
    "// Based on \"Torus Thingy 1\" by w23\n"
    "// https://www.shadertoy.com/view/llSyWD\n"
    "// License: Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License.\n"
    "\n"
    "// The original source has been modified so it runs in 60 fps on the NVidia Jetson TX2.\n"
    "// More information about changes here: https://www.ignorantus.com/xmasdemo2017/\n"
    "\n"
    "#version 310 es\n"
    "precision highp float;\n"
    "layout(location = 0) in vec2 v_texCoord;\n"
    "layout(location = 0) out vec4 outColor;\n"
    "\n"
    "layout(std140,binding = 0) uniform stuff\n"
    "{\n"
    "    float width, height;\n"
    "    float frame;\n"
    "    float colscale;\n"
    "};\n"
    "\n"
    "#define iTime frame\n"
    "\n"
    "vec3 ret_col = vec3( 0.0f ); // torus color\n"
    "vec3 h = vec3( 0.0f );       // light amount\n"
    "\n"
    "const vec3 teal   = vec3(0.3 , .7, .9);\n"
    "const vec3 orange = vec3(0.95, .5, .1);\n"
    "\n"
    "#define I_MAX  52\n"
    "#define FAR    30.0f\n"
    "\n"
    "#define PI 3.14159\n"
    "#define TAU (PI*2.0f)\n"
    "\n"
    "// Don't use long library functions, like atan().\n"
    "// Nvcc will just replace the call and don't bother with rescheduling\n"
    "// instructiions around or inside it. So the library atan2() is a\n"
    "// stallfest.\n"
    "\n"
    "// IEEE Signal Processing Magazine ( Volume: 30, Issue: 1, Jan. 2013 )\n"
    "// Full Quadrant Approximations for the Arctangent Function\n"
    "// http://ieeexplore.ieee.org/document/6375931/\n"
    "\n"
    "#define f2u( x ) floatBitsToUint( x )\n"
    "#define u2f( x ) uintBitsToFloat( x )\n"
    "\n"
    "float satan2( float y, float x )\n"
    "{\n"
    "    uint sign_mask = 0x80000000u;\n"
    "    float b = 0.596227f;\n"
    "\n"
    "    // Extract the sign bits\n"
    "    uint ux_s  = (sign_mask & f2u(x) )^sign_mask;\n"
    "    uint uy_s  = (sign_mask & f2u(y) )^sign_mask;\n"
    "\n"
    "    // Determine the quadrant offset\n"
    "    float q = float( ( ~ux_s & uy_s ) >> 29 | ux_s >> 30 );\n"
    "\n"
    "    // Calculate the arctangent in the first quadrant\n"
    "    float bxy_a = abs( b * x * y );\n"
    "    float num = bxy_a + y * y;\n"
    "    float atan_1q =  num / ( x * x + bxy_a + num );\n"
    "\n"
    "    // Translate it to the proper quadrant\n"
    "    uint uatan_2q = ((ux_s ^ uy_s) | f2u(atan_1q));\n"
    "    return (q + u2f(uatan_2q)) * PI/2.0f - PI;\n"
    "}\n"
    "\n"
    "vec2 modA( vec2 p, float count )\n"
    "{\n"
    "    float an = TAU/count;\n"
    "    float a = satan2(p.y,p.x)+an*.5;\n"
    "    a = mod(a, an)-an*.5;\n"
    "    return vec2(cos(a),sin(a))*length(p);\n"
    "}\n"
    "\n"
    "// for known s2, guess that PI/4 is close enough\n"
    "vec2 modAs2( vec2 p, float count, float s2 )\n"
    "{\n"
    "    float an = TAU/count;\n"
    "    float a = s2+an*.5 + PI/4.0f;\n"
    "    a = mod(a, an)-an*.5;\n"
    "    return vec2(cos(a),sin(a))*length(p);\n"
    "}\n"
    "\n"
    "vec2 rotate( vec2 v, float angle )\n"
    "{\n"
    "    return vec2(cos(angle)*v.x+sin(angle)*v.y,-sin(angle)*v.x+cos(angle)*v.y);\n"
    "}\n"
    "\n"
    "float scene(vec3 p)\n"
    "{\n"
    "    p.xz = rotate( p.xz, 1.57f - 0.150f * iTime );\n"
    "    p.yz = rotate( p.yz, 1.57f - 0.125f * iTime );\n"
    "    vec3 op = p;\n"
    "\n"
    "    // center spikes\n"
    "    float s2 = satan2( p.x, p.z );\n"
    "    float var = ( s2 + PI )/ TAU * 40.0f;\n"
    "    p.xz = modAs2( p.xz, 40.0f, s2 );\n"
    "    p.x -= 8.0f;\n"
    "\n"
    "    // replace mod(), hmm\n"
    "    p.xy = rotate( p.xy, iTime * 0.25f * ((int(var-0.5f)&1) == 1 ? -1. : 1.) );\n"
    "\n"
    "    vec2 q = vec2(length(p.xy)-4.,p.z);\n"
    "\n"
    "    float mind = length(q)-.05;\n"
    "\n"
    "    // orange filler\n"
    "    s2 = satan2( p.x, p.y );\n"
    "    float vir = ( s2 + PI ) / TAU;\n"
    "    var = vir * 30.0f;\n"
    "    p.xy = modAs2( p.xy, 30.0f, s2 );\n"
    "    p.x -= 4.0f;\n"
    "    q = vec2( length( p.zx ) - 0.25f, p.y );\n"
    "\n"
    "    float ming = length( q ) - 0.05f;\n"
    "\n"
    "    mind = min( mind, ming );\n"
    "\n"
    "    ret_col = mind == ming ? orange : teal;\n"
    "\n"
    "    // outer part\n"
    "    p.xz = rotate( p.xz, iTime * 1.5f * ((int(var)&1) == 1 ? -1. : 1.) * ((int(vir)&1) == 1 ? -1. : 1.) );\n"
    "\n"
    "    p.xz = modA( p.xz, 20.0f );\n"
    "    p.x -= 0.25f;\n"
    "    q = vec2( length( p.xy ) - 0.1f, p.z );\n"
    "\n"
    "    float mint = length( q ) - 0.02f;\n"
    "\n"
    "    mind = min( mind, mint );\n"
    "\n"
    "    if( mind == mint ) ret_col = vec3( 0.2f, 0.7f, 0.4f );\n"
    "\n"
    "    // dodecahedron\n"
    "    op.zx = rotate( op.zx, iTime * 0.5f );\n"
    "    op.xz = modA( op.xz, 25.0f );\n"
    "    op.x -= 6.0f;\n"
    "    op  /= 1.732; //sqrt(3.)\n"
    "\n"
    "    float b = 0.075f;\n"
    "\n"
    "    // should reconsider the op5,b stuff\n"
    "    op = abs( op );\n"
    "    vec3 op5 = 0.5f * op;\n"
    "\n"
    "    float minl;\n"
    "    minl =            max( max( op.x+op5.y-b, op.y+op5.z-b ), op.z+op5.x-b )  ; b *= .95;\n"
    "    minl = max(minl, -max( max( op.x+op5.y-b, op.y-op5.z-b ), op.z+op5.x-b ) );\n"
    "    minl = max(minl, -max( max( op.x-op5.y-b, op.y+op5.z-b ), op.z+op5.x-b ) );\n"
    "    minl = max(minl, -max( max( op.x+op5.y-b, op.y+op5.z-b ), op.z-op5.x-b ) );\n"
    "    // end dodecahedron\n"
    "\n"
    "    mind = min( mind, minl );\n"
    "\n"
    "    if( mind == minl ) ret_col = vec3( 0.5f, 0.2f, 0.8f );\n"
    "\n"
    "    h += 0.125f * vec3( 0.2f, 0.1f, 0.3f ) / ( pow( minl, 25.0f ) + 0.5f ); // dodecahedron fog\n"
    "\n"
    "    return (mind);\n"
    "}\n"
    "\n"
    "#if 0\n"
    "vec2 march(vec3 pos, vec3 dir)\n"
    "{\n"
    "    vec2 dist = vec2(0.0, 0.0);\n"
    "    int i;\n"
    "    for( i = 0; i < I_MAX+1; i++ ) {\n"
    "        vec3 p = pos + dir * dist.y;\n"
    "        dist.x = scene(p);\n"
    "        dist.y += dist.x;\n"
    "        float dinamyceps = -dist.x+(dist.y)*(1.0f/1500.0f);\n"
    "        if( dist.y > FAR || dist.x < dinamyceps || log(dist.y*dist.y/dist.x*(1.0f/100000.0f)) > 0.0f ) break;\n"
    "    }\n"
    "    return vec2( float(i), dist.y );\n"
    "}\n"
    "#endif\n"
    "\n"
    "// Will be a bit noisy, but quicker\n"
    "vec2 march(vec3 pos, vec3 dir)\n"
    "{\n"
    "    vec2 dist = vec2(0.0, 0.0);\n"
    "\n"
    "    float esc = 0.01f;\n"
    "\n"
    "    int i;\n"
    "    for( i = 0; i < I_MAX; i += 2 ) {\n"
    "        dist.x = scene( pos + dir * dist.y ); if( dist.x < esc ) break; dist.y += dist.x; if( dist.y > FAR ) break;\n"
    "        dist.x = scene( pos + dir * dist.y ); if( dist.x < esc ) break; dist.y += dist.x; if( dist.y > FAR ) break;\n"
    "    }\n"
    "\n"
    "    return vec2( float(i), dist.y );\n"
    "}\n"
    "\n"
    "vec3 camera( vec2 uv )\n"
    "{\n"
    "    float fov = 1.;\n"
    "    vec3  forw  = vec3(0.0, 0.0, -1.0);\n"
    "    vec3  right = vec3(1.0, 0.0, 0.0);\n"
    "    vec3  up    = vec3(0.0, 1.0, 0.0);\n"
    "\n"
    "    return (normalize((uv.x) * right + (uv.y) * up + fov * forw));\n"
    "}\n"
    "\n"
    "void main()\n"
    "{\n"
    "    vec2 iResolution = vec2( width, height );\n"
    "    vec2 f = (1.0f - v_texCoord) * iResolution;\n"
    "\n"
    "    float t  = iTime*.125;\n"
    "    vec3 col = vec3(0., 0., 0.);\n"
    "    vec2 R = iResolution.xy,\n"
    "          uv  = vec2(f-R/2.) / R.y;\n"
    "    vec3 dir = camera(uv);\n"
    "    vec3 pos = vec3(.0+cos(t*5.), .0+sin(t*5.)*.5, .0);\n"
    "\n"
    "    pos.z = 20.*exp(-t*5.)+10.+.5*sin(t*10.);\n"
    "    vec2 inter = (march(pos, dir));\n"
    "\n"
    "    if( inter.y <= FAR)\n"
    "        col.xyz = ret_col*(1.-inter.x*.025);\n"
    "    else\n"
    "        col = vec3( 0.0f );\n"
    "\n"
    "    col += h;\n"
    "    col *= clamp((1.5-length(uv)), .5, 1.);\n"
    "    col *= clamp((1.1-length(uv)), .0, 1.);\n"
    "\n"
    "    outColor =  vec4( col, colscale );\n"
    "}\n"
};
const char *shader_scaler = {
    "// GLSL bilinear scaler that doesn't flicker. Really.\n"
    "// Written by Nils L. Corneliusen.\n"
    "// License: CC0 1.0 Universal (CC0 1.0) Public Domain Dedication license\n"
    "// https://www.ignorantus.com/\n"
    "\n"
    "#version 310 es\n"
    "precision highp float;\n"
    "layout(location = 0) in vec2 v_texCoord;\n"
    "layout(location = 0) out vec4 outColor;\n"
    "uniform sampler2D sampid;\n"
    "uniform float tex_scrw, tex_scrh;\n"
    "void main()\n"
    "{\n"
    "    vec2 vt = v_texCoord;\n"
    "    vec2 texcoord = vec2( vt.x * tex_scrw, vt.y * tex_scrh );\n"
    "    vec2 phase = fract( texcoord );\n"
    "    phase.x = 1.0f - phase.x;\n"
    "\n"
    "    vec4 p00 = textureOffset( sampid, vt, ivec2(  0, 0 ) );\n"
    "    vec4 p10 = textureOffset( sampid, vt, ivec2( -1, 0 ) );\n"
    "    vec4 p01 = textureOffset( sampid, vt, ivec2(  0, 1 ) );\n"
    "    vec4 p11 = textureOffset( sampid, vt, ivec2( -1, 1 ) );\n"
    "\n"
    "    vec4 p0010 = mix( p00,   p10,   phase.x );\n"
    "    vec4 p0111 = mix( p01,   p11,   phase.x );\n"
    "    vec4 p     = mix( p0010, p0111, phase.y );\n"
    "\n"
    "    outColor = p;\n"
    "}\n"
};
