// Control code for the raytracer.
// Written by Nils Liaaen Corneliusen.
// License: CC0 1.0 Universal (CC0 1.0) Public Domain Dedication license
// https://www.ignorantus.com

#ifndef RAYTRACERH
#define RAYTRACERH

#include <stdint.h>

#include "vec.h"

#define OBJNUM (4*32)

#define MAXW 1920
#define MAXH 1080

#define BLOCKW 16
#define BLOCKH 16

#define MAXWR ((MAXW+BLOCKW-1)&~(BLOCKW-1))
#define MAXHR ((MAXH+BLOCKH-1)&~(BLOCKH-1))

#define BLOCKSX (MAXWR/BLOCKW)
#define BLOCKSY (MAXHR/BLOCKH)

typedef struct {
	float width, height;
	float frame;
	float colscale;

	v4 obj_pos[OBJNUM];
	v4 obj_col[OBJNUM];
	v4 plane_norm;
	v4 plane_col;
	v4 eye;
	v4 light_pos;
	v4 startpos; // precalc
	float ax; // precalc
	float ay; // precalc
	float pad0;
	float pad1;
	uint8_t probe[BLOCKSX * BLOCKSY];
} gpu_tracer;

#define PI 3.14159265f

#define SUNPOS 0

#define BLOCK_ANY 0
#define BLOCK_BG  1
#define BLOCK_SPH 2

typedef struct {
	int   orbit_center; // id of object to rotate around
	float orbit_radius;
	float orbit_tilt;   // xz..yz plane tilt, -PI..0..PI
	int   orbit_speed;  // frames for one full rotation
	float obj_radius;
	v4    obj_pos;
	v4    obj_col;
	int   obj_offset;
} sphere;

typedef struct {
	int largecnt;
	int smallcnt;
	sphere spheres[OBJNUM];
	int frame;
	int curcol;
	int width, height;
} worldinfo;

#define RAD_SUN    2.8f
#define RAD_PLANET 0.5f
#define RAD_MOON   0.2f

#define DIST_INNER (RAD_SUN+1.5f*RAD_PLANET+2.0f*RAD_MOON + 0.4f)
#define DIST_OUTER (RAD_SUN+4.0f*RAD_PLANET+2.0f*RAD_MOON + 0.5f)
#define DIST_MOON  (1.0f*RAD_PLANET+1.0f*RAD_MOON + 0.3f)

#define SPEED_INNER 615
#define SPEED_OUTER 817
#define SPEED_MOON   80

void world_init(  worldinfo *w, gpu_tracer *gt, int width, int height );
void world_timer( worldinfo *w, gpu_tracer *gt );
void world_probe( worldinfo *w, gpu_tracer *gt );

v4 rotate( float frac, float tilt, float radius );
v4 h2rgb( float h );

#endif
