// Based on "Galactic Dance" by Sinuousity
// https://www.shadertoy.com/view/XtlGWs
// License: Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License.

// The original source has been modified so it runs in 60 fps on the NVidia Jetson TX2.
// More information about changes here: https://www.ignorantus.com/pages/xmasdemo2017/

#version 310 es
precision highp float;
layout(location = 0) in vec2 v_texCoord;
layout(location = 0) out vec4 outColor;

layout(std140,binding = 0) uniform stuff
{
    float width, height;
    float frame;
    float colscale;
};

#define iTime (frame)

#define pi 3.14
#define tao 6.28

#define overbright 2.0

#define armCount 3.0
#define armRot 1.6

#define innerColor vec4(2.0,0.5,0.1,1.0)
#define outerColor vec4(0.8,0.6,1.0,1.0)
#define white vec4(1.0,1.0,1.0,1.0)


float saturate (in float f)
{
    return clamp(f,0.0,1.0);
}

vec4 drawonething( bool nozoom, float offset, float sfac, vec2 texcoord )
{
    vec2 iResolution = vec2( width, height );

    float start = 0.5f - 1.0f/sfac/2.0f;
    vec2 fragCoord = (start + texcoord/sfac) * iResolution;

//    vec2 fragCoord = ((1.0f - v_texCoord) * iResolution)*0.5f + iResolution*0.25f;

    float time = iTime + offset;

    vec2 uv = fragCoord.xy / iResolution.xy;
    if( !nozoom )
        uv = 0.5f + (uv - 0.5f) * (((sin( frame ) + 1.0f) / 2.5f) + 0.25f);

    //constant slow rotation
    float cost = cos(-time*0.2);
    float sint = sin(-time*0.2);
    mat2 trm = mat2 (cost,sint,-sint,cost);

    //scale 0.0-1.0 uv to -1.0-1.0 p
    vec2 p = uv*2.0 - 1.0;
    //apply slow rotation
    p = p * trm;

    //calc distance
    float d = length(p);

    //build arm rotation matrix
    float cosr = cos(armRot*sin(armRot*time));
    float sinr = sin(armRot*cos(armRot*time));
    mat2 rm = mat2 (cosr,sinr,-sinr,cosr);

    //calc arm rotation based on distance
    p = mix(p,p * rm,d);

    //find angle to middle
    float angle = (atan(p.y,p.x)/tao) * 0.5 + 0.5;
    //add the crinkle
    angle += sin(-time*5.0+fract(d*d*d)*10.0)*0.004;
    //calc angle in terms of arm number
    angle *= 2.0 * armCount;
    angle = fract(angle);
    //build arms & wrap the angle around 0.0 & 1.0
    float arms = abs(angle*2.0-1.0);
    //sharpen arms
    arms = pow(arms,10.0*d*d + 5.0);
    //calc radial falloff
    float bulk = 1.0 - saturate(d);
    //create glowy center
    float core = pow(bulk,9.0);
    //calc color
    vec4 color = mix(innerColor,outerColor,d*2.0);

    color = bulk * arms * color + core + bulk*0.25*mix(color,white,0.5);
    color = vec4( (overbright * color).xyz, colscale );
    color = clamp( color, 0.0f, 1.0f );
    color.a = max( max( color.r, color.g ), color.b ) * colscale;
    return color;
}

void main( void )
{
//    vec2 fragCoord = (1.0f - v_texCoord) * iResolution;

    float sfac = 1.0f;
    if( frame <= 1.3f ) {
        sfac = 32.0f;
    } else if( frame <= 3.3f ) {
        sfac = 32.0f/((frame-1.3f)*16.0f);
    } else {
        sfac = 1.0f;
    }

    vec4 pcc = drawonething( false,   0.0f, sfac, v_texCoord                          );


    if( frame > 3.3f ) {
        float rad = 0.00f + (frame-3.3f)/8.0f;
        if( rad > 0.40f ) rad = 0.40f;
        float xpos = sin( frame ) * rad;
        float ypos = cos( frame ) * rad;

        vec4 p00 = drawonething( true,  120.0f, 0.5f, v_texCoord + vec2(  xpos,  ypos ) );
        vec4 p11 = drawonething( true,  180.0f, 0.5f, v_texCoord + vec2( -xpos, -ypos ) );
        vec4 p01 = drawonething( true,  240.0f, 0.5f, v_texCoord + vec2(  xpos, -ypos ) );
        vec4 p10 = drawonething( true,  300.0f, 0.5f, v_texCoord + vec2( -xpos,  ypos ) );
        outColor = max( max( max( max( p00, p11 ), p01 ), p10 ), pcc ); // doesn't look as bad as it reads
/* sorted
        vec4 p0001 = p00.a > p01.a ? p00 : p01;
        vec4 p1011 = p10.a > p11.a ? p10 : p11;
        vec4 p     = p0001.a > p1011.a ? p0001 : p1011;
        outColor = p.a > pcc.a ? p : pcc;
*/
    } else {
        outColor = pcc;
    }

}
