// Based on "Glow City" by mhnewman
// https://www.shadertoy.com/view/XlsyWB
// License: Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License.

// The original source has been modified so it runs in 60 fps on the NVidia Jetson TX2.
// More information about changes here: https://www.ignorantus.com/pages/xmasdemo2017/

#version 310 es
precision highp float;
layout(location = 0) in vec2 v_texCoord;
layout(location = 0) out vec4 outColor;

layout(std140,binding = 0) uniform stuff
{
    float width, height;
    float frame;
    float colscale;
};

#define iTime (frame)

vec2 iResolution = vec2( 1920.0f, 1080.0f );
vec3 iMouse = vec3( 0.0f, 0.0f, 0.0f );

// Grid marching from iq's Grid of Cylinders
// https://www.shadertoy.com/view/4dSGW1

const float streetDistance = 0.6;
const vec3 streetColor = vec3(4.0, 8.0, 10.0);

const float fogDensity = 0.5;
const float fogDistance = 4.0;
const vec3 fogColor = vec3(0.34, 0.37, 0.4);

const float windowSize = 0.075;
const float windowSpead = 1.0;

const vec3 windowColor = vec3(0.1, 0.2, 1.0);

const float beaconProb = 0.0006;
const float beaconFreq = 0.6;
const vec3 beaconColor = vec3(1.5, 0.2, 0.0);


const float tau = 6.283185;

float hash1( vec2 p2 )
{
    p2 = fract(p2 * vec2(5.3983, 5.4427));
    p2 += dot(p2.yx, p2.xy + vec2(21.5351, 14.3137));
    return fract(p2.x * p2.y * 95.4337);

}

float hash1( vec2 p2, float p )
{
    vec3 p3 = fract(vec3(5.3983 * p2.x, 5.4427 * p2.y, 6.9371 * p));
    p3 += dot(p3, p3.yzx + 19.19);
    return fract((p3.x + p3.y) * p3.z);
}

vec2 hash2( vec2 p2 )
{
    return vec2( ( sin( p2.x * p2.y ) + 1.0f ) / 2.0f, cos( p2.x*p2.y ) + 1.0f );
}

vec2 hash2( vec2 p2, float p )
{
    return vec2( sin( p2.x ) + 1.0f, cos( p2.y ) * p );
}

vec3 hash3( vec2 p2 )
{
    vec3 p3 = fract(vec3(p2.xyx) * vec3(5.3983, 5.4427, 6.9371));
    p3 += dot(p3, p3.yxz + 19.19);
    return fract((p3.xxy + p3.yzz) * p3.zyx);
}

float noise1( vec2 p )
{
    return ( (sin( p.x * 5.78f ) * cos( p.y * 4.23f ) ) + 1.0f ) / 2.0f;
}

vec4 castRay( vec3 eye, vec3 ray )
{
    vec2 block = floor(eye.xy);
    vec3 ri = 1.0 / ray;
    vec3 rs = sign(ray);
    vec3 side = 0.5 + 0.5 * rs;
    vec2 ris = ri.xy * rs.xy;
    vec2 dis = (block - eye.xy + 0.5 + rs.xy * 0.5) * ri.xy;

    float beacon = 0.0;

    for (int i = 0; i < 100; ++i) {

        float height = (0.5 + hash1(block)) * (2.0 + 4.0 * pow(noise1(0.1 * block), 2.5)) * 0.80f + 2.0f;

        float dist = 500.0;
        float face = 0.0;

        vec3 lo = vec3( block + 0.01f + 0.3f * hash2(block, 0.0f), 0.0f );
        vec3 hi = vec3( block + 0.69f + 0.3f * hash2(block, 0.5f), height );

        vec3 wall = mix(hi, lo, side);
        vec3 tw = (wall - eye) * ri;

        vec3 dim3 = step(tw.zxy, tw) * step(tw.yzx, tw);

        float maxT = dot(dim3, tw);
        float maxFace = 1.0 - dim3.z;

        vec3 p0 = eye + maxT * ray;
        dim3 += step(lo, p0) * step(p0, hi);
        if (dim3.x * dim3.y * dim3.z > 0.5 && maxT < dist) {
            dist = maxT;
            face = maxFace;
        }

        vec2 h = hash2( block );
        if (h.x < beaconProb * (height - 2.0f) * (height - 2.0f) ) {
            vec3 center = vec3(block + 0.5, height + 0.2);
            float t = dot(center - eye, ray);
            if (t < dist) {
                vec3 p = eye + t * ray;
                float fog = (exp(-p.z / fogDistance) - exp(-eye.z / fogDistance)) / ray.z;
                fog = exp(fogDensity * fog);

                t = distance(center, p);
                fog *= smoothstep(1.0, 0.5, cos(tau * (beaconFreq * iTime + h.y)));
                beacon += fog * pow(clamp(1.0 - 2.0 * t, 0.0, 1.0), 2.0);
            }
        }

        if (dist < 400.0) { // 400.0f
            return vec4(dist, beacon, face, 1.0);
        }

        float t = eye.z * ri.z;
        vec3 p = eye - t * ray;
        vec2 g = p.xy - block;
        if (g.x > 0.0 && g.x < 1.0 && g.y > 0.0 && g.y < 1.0) {
            return vec4(-t, beacon, 0.0, 1.0);
        }

        vec2 dim = step(dis.xy, dis.yx);
        dis += dim * ris;
        block += dim * rs.xy;
    }

    if (ray.z < 0.0) {
        return vec4(-eye.z * ri.z, beacon, 0.0, 1.0);
    }

    return vec4(0.0, beacon, 0.0, 0.0);
}

void main( void )
{
    vec2 fragCoord = (1.0f - v_texCoord) * iResolution;

    vec2 m = vec2(0.03 * iTime, 0.8);
    if (iMouse.z > 0.0)
        m = iMouse.xy / iResolution.xy;
    m *= tau * vec2(1.0, 0.25);

    vec3 center = vec3(6.0 * iTime, 0.5, 3.0);
    float dist = 20.0;
    vec3 eye = center + vec3(dist * sin(m.x) * sin(m.y), dist * cos(m.x) * sin(m.y), dist * cos(m.y));
    float zoom = 3.0;

    vec3 forward = normalize(center - eye);
    vec3 right = normalize(cross(forward, vec3(0.0, 0.0, 1.0)));
    vec3 up = cross(right, forward);
    vec2 xy = 2.0 * fragCoord - iResolution.xy;
    zoom *= iResolution.y;
    vec3 ray = normalize(xy.x * right + xy.y * up + zoom * forward);

    vec4 res = castRay(eye, ray);
    if (res.w < 0.5) {
        outColor = vec4(fogColor + res.y * beaconColor, colscale);
        return;
    }

    vec3 p = eye + res.x * ray;

    vec2 block = floor(p.xy);
    vec3 window = floor(p / windowSize);
    float x = hash1(block, window.x);
    float y = hash1(block, window.y);
    float z = hash1(block, window.z);
    vec3 color = windowColor + windowSpead * (hash3(block) - 0.5);
    color *= smoothstep(0.1, 0.9, fract(2.5 * (x * y * z)));
    color += streetColor * exp(-p.z / streetDistance);

    vec3 level = 0.25 * streetColor * exp(-4.0 * p.z / streetDistance);
    color = clamp(mix(level, color, res.z), 0.0, 1.0);

    float fog = (exp(-p.z / fogDistance) - exp(-eye.z / fogDistance)) / ray.z;
    fog = exp(fogDensity * fog);
    color = mix(fogColor, color, fog);

    color += res.y * beaconColor;
    color += pow(res.y, 2.0);

    outColor = vec4(color, colscale);
}
