// Win32 OpenGL support functions.
// Written by Sjur Julin.
// License: CC0 1.0 Universal (CC0 1.0) Public Domain Dedication license
// https://www.in-a-vision.com

#include <stdio.h>
#include <stdint.h>
#include <stdbool.h>

#include <Windows.h>
#include <GL/glew.h>
#include <GL/GL.h>
#include <GL/wglext.h>

#include "resource.h"
#include "glstuff.h"

HINSTANCE hInst;
HWND hWnd, hWndFrame;
HDC hDC;
HGLRC hGLRC;

extern int quit;
static int keys[256];

double PCFreq = 0.0;
__int64 CounterStart = 0;

double PerfCounter( void )
{
    LARGE_INTEGER li;

	if( PCFreq == 0.0) {
		if(!QueryPerformanceFrequency(&li)) {
			printf( "doh" );
			exit( 1 );
		}
		PCFreq = (double)li.QuadPart;
		QueryPerformanceCounter(&li);
		CounterStart = li.QuadPart;
	}

    QueryPerformanceCounter(&li);
	return (li.QuadPart-CounterStart)/PCFreq;
}

static LRESULT CALLBACK WndProc(HWND hWnd, UINT Msg, WPARAM wParam, LPARAM lParam) {
//	GL7Context *g = (GL7Context*)GetWindowLongPtr(hWnd, 0);
	switch (Msg) {
	case WM_KEYDOWN:
		keys[wParam] = 1;
		break;
	case WM_KEYUP:
		keys[wParam] = 0;
		break;
	case WM_DESTROY:
		quit=1;
		break;
	case WM_KILLFOCUS:
		memset(keys, 0x00, 256); // release all keys
		break;
	case WM_WINDOWPOSCHANGED: { // for resizing
		RECT r;
		GetClientRect(hWnd, &r);
		//w = r.right-r.left;
		//h = r.bottom-r.top;
		// call routine to resize opengl!
	}	break;
	case WM_EXITSIZEMOVE:
		KillTimer(hWnd, 1234);
		break;
	case WM_SETCURSOR:
		if(LOWORD(lParam)==HTCLIENT) { SetCursor(0); return 1; }
		break;
	default:
 		return DefWindowProcW(hWnd, Msg, wParam, lParam);
	}
	if(keys[VK_ESCAPE])
		quit=1;

	return 0;
}

static int CreateWin(int w, int h) {

	int x = GetSystemMetrics(SM_CXSCREEN)/2 - w/2;
	int y = GetSystemMetrics(SM_CYSCREEN)/2 - h/2;

	hInst = GetModuleHandleW(0);

	WNDCLASSW wc;
	memset(&wc, 0, sizeof(wc));
	wc.style = CS_CLASSDC;
//	wc.style = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
	wc.lpfnWndProc = WndProc;
	wc.cbWndExtra = 0;
	wc.hIcon = (HICON)LoadIcon(hInst, (const char*)IDI_MAINICON);
	wc.hInstance = hInst;
	wc.hCursor = 0;
	wc.lpszClassName = L"XmasWin32Class";
	if(!RegisterClassW(&wc)) return -1;

	// adjust the window size and show at InitDevice time
	hWnd = CreateWindowExW(0, wc.lpszClassName, L"Xmasdemo", WS_POPUP|WS_VISIBLE, x, y, w, h, 0, 0, hInst, 0);
	if(!hWnd) return -1;

	hDC = GetDC(hWnd);
	if(!hDC) return -1;

//	RECT r = { 0, 0, w, h };
//	AdjustWindowRect(&r, WS_OVERLAPPEDWINDOW, 0);
//	const UINT flags = SWP_NOMOVE | SWP_NOZORDER | SWP_SHOWWINDOW;
//	if(!SetWindowPos(hWnd, 0, 0, 0, r.right-r.left, r.bottom-r.top, flags)) return -1;

	static const PIXELFORMATDESCRIPTOR pfd = {
		sizeof(PIXELFORMATDESCRIPTOR), 1,							// size, version
		PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER, // flags
		PFD_TYPE_RGBA, 32,											// pixeltype, colorbits
		0,0,0,0,0,0,0,0,0,0,0,0,0,									// bit patterns
		24,8,0,														// depthbits, stencilbits, auxbuffers
		PFD_MAIN_PLANE,												// layertype
		0,0,0,0														// reserved, masks
	};

	uint32_t pf = ChoosePixelFormat(hDC, &pfd);
	if(!SetPixelFormat(hDC, pf, &pfd)) return -1;
	if(!(hGLRC=wglCreateContext(hDC))) return -1;
	if(!wglMakeCurrent(hDC, hGLRC)) return -1;

	return 0;
}

bool WGLExtensionSupported(const char *name) {
	PFNWGLGETEXTENSIONSSTRINGEXTPROC func =
		(PFNWGLGETEXTENSIONSSTRINGEXTPROC) wglGetProcAddress("wglGetExtensionsStringEXT");
	if(strstr(func(), name)) return true;
	return false;
}

bool initOpenGL( bool display, int width, int height ) {

	(void)display;

	if(CreateWin(width, height)) {
		puts("Failed to create window");
		return false;
	}

	GLint res = glewInit();
	if(res != GLEW_OK) {
		printf("GLEW initialization failed: %s\n", glewGetErrorString(res));
		return false;
	}

	// Enable VSYNC in OpenGL https://stackoverflow.com/questions/589064/how-to-enable-vertical-sync-in-opengl#589232
	if(WGLExtensionSupported("WGL_EXT_swap_control")) {
		PFNWGLSWAPINTERVALEXTPROC wglSwapIntervalEXT = (PFNWGLSWAPINTERVALEXTPROC) wglGetProcAddress("wglSwapIntervalEXT");	
		PFNWGLGETSWAPINTERVALEXTPROC wglGetSwapIntervalEXT = (PFNWGLGETSWAPINTERVALEXTPROC) wglGetProcAddress("wglGetSwapIntervalEXT");
		int v = wglGetSwapIntervalEXT();
		wglSwapIntervalEXT(-1);
		printf("VSYNC: %d (was %d)\n", wglGetSwapIntervalEXT(), v);
	} else puts("VSYNC: N/A");

	return true;
}

void postRender() {

	SwapBuffers(hDC);

	MSG msg;
	if(PeekMessage(&msg, NULL, 0U, 0U, PM_REMOVE)) {
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}
}

void shutdownOpenGL() {

	if(hGLRC) {
		wglMakeCurrent(0, 0);
		wglDeleteContext(hGLRC);
		hGLRC = 0;
	}

	if(hWnd) {
		if(hDC)
			ReleaseDC(hWnd, hDC);
		hDC = 0;
		DestroyWindow(hWnd);
		hWnd = 0;
		UnregisterClass("XmasWin32Class", hInst);
	}
}
